@@ -1,7 +1,7 @@
 /* maptermlist.h
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -87,6 +87,14 @@ class MapTermList : public TermList {
 	    return NULL;
 	}
 
+	TermList * skip_to(const std::string & term) {
+	    while (it != it_end && it->first < term) {
+		++it;
+	    }
+	    started = true;
+	    return NULL;
+	}
+
 	// True if we're off the end of the list
 	bool at_end() const {
 	    Assert(started);
