@@ -47,7 +47,6 @@
 #include "brass_values.h"
 #include "debuglog.h"
 #include "io_utils.h"
-#include "omtime.h"
 #include "pack.h"
 #include "remoteconnection.h"
 #include "replication.h"
@@ -518,8 +517,7 @@ BrassDatabase::get_database_write_lock(bool creating)
 }
 
 void
-BrassDatabase::send_whole_database(RemoteConnection & conn,
-				   const OmTime & end_time)
+BrassDatabase::send_whole_database(RemoteConnection & conn, double end_time)
 {
     LOGCALL_VOID(DB, "BrassDatabase::send_whole_database", conn | end_time);
 
@@ -576,7 +574,6 @@ BrassDatabase::write_changesets_to_fd(int fd,
     }
 
     RemoteConnection conn(-1, fd, string());
-    OmTime end_time;
 
     // While the starting revision number is less than the latest revision
     // number, look for a changeset, and write it.
@@ -592,7 +589,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 	    if (whole_db_copies_left == 0) {
 		conn.send_message(REPL_REPLY_FAIL,
 				  "Database changing too fast",
-				  end_time);
+				  0.0);
 		return;
 	    }
 	    whole_db_copies_left--;
@@ -601,7 +598,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 	    start_rev_num = get_revision_number();
 	    start_uuid = get_uuid();
 
-	    send_whole_database(conn, end_time);
+	    send_whole_database(conn, 0.0);
 	    if (info != NULL)
 		++(info->fullcopy_count);
 
@@ -616,7 +613,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 		string buf;
 		needed_rev_num = get_revision_number();
 		pack_uint(buf, needed_rev_num);
-		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, 0.0);
 		if (info != NULL && start_rev_num == needed_rev_num)
 		    info->changed = true;
 	    } else {
@@ -630,7 +627,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 
 		string buf;
 		pack_uint(buf, start_rev_num + 1);
-		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, 0.0);
 		need_whole_db = true;
 	    }
 	} else {
@@ -665,7 +662,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 		// FIXME - there is a race condition here - the file might get
 		// deleted between the file_exists() test and the access to
 		// send it.
-		conn.send_file(REPL_REPLY_CHANGESET, changes_name, end_time);
+		conn.send_file(REPL_REPLY_CHANGESET, changes_name, 0.0);
 		start_rev_num = changeset_end_rev_num;
 		if (info != NULL) {
 		    ++(info->changeset_count);
@@ -679,7 +676,7 @@ BrassDatabase::write_changesets_to_fd(int fd,
 	    }
 	}
     }
-    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), end_time);
+    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), 0.0);
 }
 
 void
