@@ -49,7 +49,6 @@
 #include "chert_values.h"
 #include "debuglog.h"
 #include "io_utils.h"
-#include "omtime.h"
 #include "pack.h"
 #include "remoteconnection.h"
 #include "replicate_utils.h"
@@ -513,8 +512,7 @@ ChertDatabase::get_database_write_lock(bool creating)
 }
 
 void
-ChertDatabase::send_whole_database(RemoteConnection & conn,
-				   const OmTime & end_time)
+ChertDatabase::send_whole_database(RemoteConnection & conn, double end_time)
 {
     LOGCALL_VOID(DB, "ChertDatabase::send_whole_database", conn | end_time);
 
@@ -571,7 +569,6 @@ ChertDatabase::write_changesets_to_fd(int fd,
     }
 
     RemoteConnection conn(-1, fd, string());
-    OmTime end_time;
 
     // While the starting revision number is less than the latest revision
     // number, look for a changeset, and write it.
@@ -587,7 +584,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 	    if (whole_db_copies_left == 0) {
 		conn.send_message(REPL_REPLY_FAIL,
 				  "Database changing too fast",
-				  end_time);
+				  0.0);
 		return;
 	    }
 	    whole_db_copies_left--;
@@ -596,7 +593,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 	    start_rev_num = get_revision_number();
 	    start_uuid = get_uuid();
 
-	    send_whole_database(conn, end_time);
+	    send_whole_database(conn, 0.0);
 	    if (info != NULL)
 		++(info->fullcopy_count);
 
@@ -611,7 +608,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 		string buf;
 		needed_rev_num = get_revision_number();
 		pack_uint(buf, needed_rev_num);
-		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, 0.0);
 		if (info != NULL && start_rev_num == needed_rev_num)
 		    info->changed = true;
 	    } else {
@@ -625,7 +622,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 
 		string buf;
 		pack_uint(buf, start_rev_num + 1);
-		conn.send_message(REPL_REPLY_DB_FOOTER, buf, end_time);
+		conn.send_message(REPL_REPLY_DB_FOOTER, buf, 0.0);
 		need_whole_db = true;
 	    }
 	} else {
@@ -660,7 +657,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 		// FIXME - there is a race condition here - the file might get
 		// deleted between the file_exists() test and the access to
 		// send it.
-		conn.send_file(REPL_REPLY_CHANGESET, changes_name, end_time);
+		conn.send_file(REPL_REPLY_CHANGESET, changes_name, 0.0);
 		start_rev_num = changeset_end_rev_num;
 		if (info != NULL) {
 		    ++(info->changeset_count);
@@ -674,7 +671,7 @@ ChertDatabase::write_changesets_to_fd(int fd,
 	    }
 	}
     }
-    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), end_time);
+    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), 0.0);
 }
 
 void
