@@ -523,7 +523,7 @@ ChertValueManager::get_value_stats(Xapian::valueno slot) const
 void
 ChertValueManager::get_value_stats(Xapian::valueno slot, ValueStats & stats) const
 {
-    LOGCALL_VOID(DB, "ChertValueManager::get_value_stats", slot | "[stats]");
+    LOGCALL_VOID(DB, "ChertValueManager::get_value_stats", slot | Literal("[stats]"));
     // Invalidate the cache first in case an exception is thrown.
     mru_valno = Xapian::BAD_VALUENO;
 
