@@ -1,7 +1,7 @@
 /* flint_cursor.cc: Btree cursor implementation
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,19 +21,19 @@
 
 #include <config.h>
 
+#include "flint_cursor.h"
+
 #include "safeerrno.h"
 
 #include <xapian/error.h>
 
-#include "flint_cursor.h"
+#include "debuglog.h"
 #include "flint_table.h"
-#include "flint_btreeutil.h"
 #include "omassert.h"
-#include "omdebug.h"
 
-#ifdef XAPIAN_DEBUG_VERBOSE
+#ifdef XAPIAN_DEBUG_LOG
 static string
-hex_encode(const string & input)
+hex_display_encode(const string & input)
 {
     const char * table = "0123456789abcdef";
     string result;
@@ -55,8 +55,10 @@ FlintCursor::FlintCursor(FlintTable *B_)
 	  is_after_end(false),
 	  tag_status(UNREAD),
 	  B(B_),
+	  version(B_->cursor_version),
 	  level(B_->level)
 {
+    B->cursor_created_since_last_modification = true;
     C = new Cursor_[level + 1];
 
     for (int j = 0; j < level; j++) {
@@ -67,6 +69,36 @@ FlintCursor::FlintCursor(FlintTable *B_)
     C[level].p = B->C[level].p;
 }
 
+void
+FlintCursor::rebuild()
+{
+    int new_level = B->level;
+    if (new_level <= level) {
+	for (int i = 0; i < new_level; i++) {
+	    C[i].n = BLK_UNUSED;
+	}
+	for (int j = new_level; j < level; ++j) {
+	    delete C[j].p;
+	}
+    } else {
+	Cursor_ * old_C = C;
+	C = new Cursor_[new_level + 1];
+	for (int i = 0; i < level; i++) {
+	    C[i].p = old_C[i].p;
+	    C[i].n = BLK_UNUSED;
+	}
+	delete old_C;
+	for (int j = level; j < new_level; j++) {
+	    C[j].p = new byte[B->block_size];
+	    C[j].n = BLK_UNUSED;
+	}
+    }
+    level = new_level;
+    C[level].n = B->C[level].n;
+    C[level].p = B->C[level].p;
+    version = B->cursor_version;
+}
+
 FlintCursor::~FlintCursor()
 {
     // Use the value of level stored in the cursor rather than the
@@ -80,23 +112,24 @@ FlintCursor::~FlintCursor()
 bool
 FlintCursor::prev()
 {
-    DEBUGCALL(DB, bool, "FlintCursor::prev", "");
-    Assert(B->level <= level);
+    LOGCALL(DB, bool, "FlintCursor::prev", NO_ARGS);
     Assert(!is_after_end);
-
-    if (!is_positioned) {
-	// We've read the last key and tag, and we're now not positioned.
-	// Simple fix - seek to the current key, and then it's as if we
-	// read the key but not the tag.
-	(void)find_entry(current_key);
-	tag_status = UNREAD;
-    }
-
-    if (tag_status != UNREAD) {
+    if (B->cursor_version != version || !is_positioned) {
+	// Either the cursor needs rebuilding (in which case find_entry() will
+	// call rebuild() and then reposition the cursor), or we've read the
+	// last key and tag, and we're now not positioned (in which case we
+	// seek to the current key, and then it's as if we read the key but not
+	// the tag).
+	if (!find_entry(current_key)) {
+	    // Exact entry was no longer there after rebuild(), and we've
+	    // automatically ended up on the entry before it.
+	    RETURN(true);
+	}
+    } else if (tag_status != UNREAD) {
 	while (true) {
 	    if (! B->prev(C, 0)) {
 		is_positioned = false;
-		return false;
+		RETURN(false);
 	    }
 	    if (Item_(C[0].p, C[0].c).component_of() == 1) {
 		break;
@@ -107,7 +140,7 @@ FlintCursor::prev()
     while (true) {
 	if (! B->prev(C, 0)) {
 	    is_positioned = false;
-	    return false;
+	    RETURN(false);
 	}
 	if (Item_(C[0].p, C[0].c).component_of() == 1) {
 	    break;
@@ -116,16 +149,22 @@ FlintCursor::prev()
     get_key(&current_key);
     tag_status = UNREAD;
 
-    DEBUGLINE(DB, "Moved to entry: key=`" << hex_encode(current_key) << "'");
-    return true;
+    LOGLINE(DB, "Moved to entry: key=" << hex_display_encode(current_key));
+    RETURN(true);
 }
 
 bool
 FlintCursor::next()
 {
-    DEBUGCALL(DB, bool, "FlintCursor::next", "");
-    Assert(B->level <= level);
+    LOGCALL(DB, bool, "FlintCursor::next", NO_ARGS);
     Assert(!is_after_end);
+    if (B->cursor_version != version) {
+	// find_entry() will call rebuild().
+	(void)find_entry(current_key);
+	// If the key was found, we're now pointing to it, otherwise we're
+	// pointing to the entry before.  Either way, we now want to move to
+	// the next key.
+    }
     if (tag_status == UNREAD) {
 	while (true) {
 	    if (! B->next(C, 0)) {
@@ -141,21 +180,23 @@ FlintCursor::next()
 
     if (!is_positioned) {
 	is_after_end = true;
-	return false;
+	RETURN(false);
     }
 
     get_key(&current_key);
     tag_status = UNREAD;
 
-    DEBUGLINE(DB, "Moved to entry: key=`" << hex_encode(current_key) << "'");
-    return true;
+    LOGLINE(DB, "Moved to entry: key=" << hex_display_encode(current_key));
+    RETURN(true);
 }
 
 bool
 FlintCursor::find_entry(const string &key)
 {
-    DEBUGCALL(DB, bool, "FlintCursor::find_entry", key);
-    Assert(B->level <= level);
+    LOGCALL(DB, bool, "FlintCursor::find_entry", key);
+    if (B->cursor_version != version) {
+	rebuild();
+    }
 
     is_after_end = false;
 
@@ -193,15 +234,17 @@ done:
 	get_key(&current_key);
     tag_status = UNREAD;
 
-    DEBUGLINE(DB, "Found entry: key=`" << hex_encode(current_key) << "'");
+    LOGLINE(DB, "Found entry: key=" << hex_display_encode(current_key));
     RETURN(found);
 }
 
 bool
 FlintCursor::find_entry_ge(const string &key)
 {
-    DEBUGCALL(DB, bool, "FlintCursor::find_entry_ge", key);
-    Assert(B->level <= level);
+    LOGCALL(DB, bool, "FlintCursor::find_entry_ge", key);
+    if (B->cursor_version != version) {
+	rebuild();
+    }
 
     is_after_end = false;
 
@@ -232,7 +275,7 @@ FlintCursor::find_entry_ge(const string &key)
     }
     tag_status = UNREAD;
 
-    DEBUGLINE(DB, "Found entry: key=`" << hex_encode(current_key) << "'");
+    LOGLINE(DB, "Found entry: key=" << hex_display_encode(current_key));
     RETURN(found);
 }
 
@@ -248,7 +291,7 @@ FlintCursor::get_key(string * key) const
 bool
 FlintCursor::read_tag(bool keep_compressed)
 {
-    DEBUGCALL(DB, bool, "FlintCursor::read_tag", keep_compressed);
+    LOGCALL(DB, bool, "FlintCursor::read_tag", keep_compressed);
     if (tag_status == UNREAD) {
 	Assert(B->level <= level);
 	Assert(is_positioned);
@@ -263,9 +306,9 @@ FlintCursor::read_tag(bool keep_compressed)
 	// cursor ends up on the next key.
 	is_positioned = B->next(C, 0);
 
-	DEBUGLINE(DB, "tag=`" << hex_encode(current_tag) << "'");
+	LOGLINE(DB, "tag=" << hex_display_encode(current_tag));
     }
-    return (tag_status == COMPRESSED);
+    RETURN(tag_status == COMPRESSED);
 }
 
 bool
