@@ -1,7 +1,7 @@
 /** @file flint_spelling.h
  * @brief Spelling correction data for a flint database.
  */
-/* Copyright (C) 2007,2009 Olly Betts
+/* Copyright (C) 2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,16 +29,16 @@
 #include <map>
 #include <set>
 #include <string>
-#include <string.h> // For memcpy() and memcmp().
+#include <cstring> // For memcpy() and memcmp().
 
-struct fragment {
+struct F_fragment {
     char data[4];
 
     // Default constructor.
-    fragment() { }
+    F_fragment() { }
 
     // Allow implicit conversion.
-    fragment(char data_[4]) { memcpy(data, data_, 4); }
+    F_fragment(char data_[4]) { std::memcpy(data, data_, 4); }
 
     char & operator[] (unsigned i) { return data[i]; }
     const char & operator[] (unsigned i) const { return data[i]; }
@@ -48,15 +48,15 @@ struct fragment {
     }
 };
 
-inline bool operator<(const fragment &a, const fragment &b) {
-    return memcmp(a.data, b.data, 4) < 0;
+inline bool operator<(const F_fragment &a, const F_fragment &b) {
+    return std::memcmp(a.data, b.data, 4) < 0;
 }
 
 class FlintSpellingTable : public FlintTable {
-    void toggle_fragment(fragment frag, const std::string & word);
+    void toggle_fragment(F_fragment frag, const std::string & word);
 
     std::map<std::string, Xapian::termcount> wordfreq_changes;
-    std::map<fragment, std::set<std::string> > termlist_deltas;
+    std::map<F_fragment, std::set<std::string> > termlist_deltas;
 
   public:
     /** Create a new FlintSpellingTable object.
@@ -68,7 +68,7 @@ class FlintSpellingTable : public FlintTable {
      *  @param readonly		true if we're opening read-only, else false.
      */
     FlintSpellingTable(const std::string & dbdir, bool readonly)
-	: FlintTable(dbdir + "/spelling.", readonly, Z_DEFAULT_STRATEGY, true) { }
+	: FlintTable("spelling", dbdir + "/spelling.", readonly, Z_DEFAULT_STRATEGY, true) { }
 
     // Merge in batched-up changes.
     void merge_changes();
@@ -98,9 +98,9 @@ class FlintSpellingTable : public FlintTable {
 	FlintTable::set_block_size(blocksize);
     }
 
-    void commit(flint_revision_number_t revision) {
+    void flush_db() {
 	merge_changes();
-	FlintTable::commit(revision);
+	FlintTable::flush_db();
     }
 
     void cancel() {
@@ -146,7 +146,9 @@ class FlintSpellingTermList : public TermList {
 
     Xapian::termcount get_collection_freq() const;
 
-    TermList *next();
+    TermList * next();
+
+    TermList * skip_to(const std::string & term);
 
     bool at_end() const;
 
