@@ -2,8 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
- * Copyright 2006 Richard Boulton
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,14 +23,15 @@
 
 #include <config.h>
 
-#include "omdebug.h"
-
-#include "emptypostlist.h"
-#include "expandweight.h"
 #include "inmemory_database.h"
+
+#include "debuglog.h"
+
+#include "expandweight.h"
 #include "inmemory_document.h"
 #include "inmemory_alltermslist.h"
-#include "utils.h"
+#include "str.h"
+#include "valuestats.h"
 
 #include <string>
 #include <vector>
@@ -76,15 +77,15 @@ InMemoryDoc::add_posting(const InMemoryTermEntry & post)
 //////////////
 
 InMemoryPostList::InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_,
-				   const InMemoryTerm & imterm)
-	: pos(imterm.docs.begin()),
+				   const InMemoryTerm & imterm,
+				   const std::string & term_)
+	: LeafPostList(term_),
+	  pos(imterm.docs.begin()),
 	  end(imterm.docs.end()),
 	  termfreq(imterm.term_freq),
 	  started(false),
 	  db(db_)
 {
-    // InMemoryPostLists cannot be empty
-    Assert(pos != end);
     while (pos != end && !pos->valid) ++pos;
 }
 
@@ -97,16 +98,16 @@ InMemoryPostList::get_termfreq() const
 Xapian::docid
 InMemoryPostList::get_docid() const
 {
-    //DebugMsg(tname << ".get_docid()");
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     Assert(!at_end());
-    //DebugMsg(" = " << (*pos).did << endl);
     return (*pos).did;
 }
 
 PostList *
 InMemoryPostList::next(Xapian::weight /*w_min*/)
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     if (started) {
 	Assert(!at_end());
 	++pos;
@@ -120,7 +121,7 @@ InMemoryPostList::next(Xapian::weight /*w_min*/)
 PostList *
 InMemoryPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 {
-    //DebugMsg(tname << ".skip_to(" << did << ")" << endl);
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     // FIXME - see if we can make more efficient, perhaps using better
     // data structure.  Note, though, that a binary search of
     // the remaining list may NOT be a good idea (search time is then
@@ -138,24 +139,27 @@ InMemoryPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 bool
 InMemoryPostList::at_end() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return (pos == end);
 }
 
 string
 InMemoryPostList::get_description() const
 {
-    return "InMemoryPostList " + om_tostring(termfreq);
+    return "InMemoryPostList " + str(termfreq);
 }
 
-Xapian::doclength
+Xapian::termcount
 InMemoryPostList::get_doclength() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return db->get_doclength(get_docid());
 }
 
 PositionList *
 InMemoryPostList::read_position_list()
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     mypositions.set_data(pos->positions);
     return &mypositions;
 }
@@ -163,12 +167,14 @@ InMemoryPostList::read_position_list()
 PositionList *
 InMemoryPostList::open_position_list() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return new InMemoryPositionList(pos->positions);
 }
 
 Xapian::termcount
 InMemoryPostList::get_wdf() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return (*pos).wdf;
 }
 
@@ -183,13 +189,14 @@ InMemoryTermList::InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDat
 	: pos(doc.terms.begin()), end(doc.terms.end()), terms(doc.terms.size()),
 	  started(false), db(db_), did(did_), document_length(len)
 {
-    DEBUGLINE(DB, "InMemoryTermList::InMemoryTermList(): " <<
-	          terms << " terms starting from " << pos->tname);
+    LOGLINE(DB, "InMemoryTermList::InMemoryTermList(): " <<
+	        terms << " terms starting from " << pos->tname);
 }
 
 Xapian::termcount
 InMemoryTermList::get_wdf() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     Assert(!at_end());
     return (*pos).wdf;
@@ -198,6 +205,7 @@ InMemoryTermList::get_wdf() const
 Xapian::doccount
 InMemoryTermList::get_termfreq() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     Assert(!at_end());
 
@@ -207,12 +215,14 @@ InMemoryTermList::get_termfreq() const
 Xapian::termcount
 InMemoryTermList::get_approx_size() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return terms;
 }
 
 void
 InMemoryTermList::accumulate_stats(Xapian::Internal::ExpandStats & stats) const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     Assert(!at_end());
     stats.accumulate(InMemoryTermList::get_wdf(), document_length,
@@ -223,6 +233,7 @@ InMemoryTermList::accumulate_stats(Xapian::Internal::ExpandStats & stats) const
 string
 InMemoryTermList::get_termname() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     Assert(!at_end());
     return (*pos).tname;
@@ -231,6 +242,7 @@ InMemoryTermList::get_termname() const
 TermList *
 InMemoryTermList::next()
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     if (started) {
 	Assert(!at_end());
 	pos++;
@@ -240,9 +252,24 @@ InMemoryTermList::next()
     return NULL;
 }
 
+TermList *
+InMemoryTermList::skip_to(const string & term)
+{
+    if (rare(db->is_closed()))
+	InMemoryDatabase::throw_database_closed();
+
+    while (pos != end && pos->tname < term) {
+	++pos;
+    }
+
+    started = true;
+    return NULL;
+}
+
 bool
 InMemoryTermList::at_end() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
     return (pos == end);
 }
@@ -250,12 +277,14 @@ InMemoryTermList::at_end() const
 Xapian::termcount
 InMemoryTermList::positionlist_count() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return db->positionlist_count(did, (*pos).tname);
 }
 
 Xapian::PositionIterator
 InMemoryTermList::positionlist_begin() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return Xapian::PositionIterator(db->open_position_list(did, (*pos).tname));
 }
 
@@ -264,28 +293,31 @@ InMemoryTermList::positionlist_begin() const
 /////////////////////////////
 
 InMemoryAllDocsPostList::InMemoryAllDocsPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_)
-	: did(0), db(db_)
+	: LeafPostList(std::string()), did(0), db(db_)
 {
 }
 
 Xapian::doccount
 InMemoryAllDocsPostList::get_termfreq() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return db->totdocs;
 }
 
 Xapian::docid
 InMemoryAllDocsPostList::get_docid() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(did > 0);
     Assert(did <= db->termlists.size());
     Assert(db->termlists[did - 1].is_valid);
     return did;
 }
 
-Xapian::doclength
+Xapian::termcount
 InMemoryAllDocsPostList::get_doclength() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return db->get_doclength(did);
 }
 
@@ -310,6 +342,7 @@ InMemoryAllDocsPostList::open_position_list() const
 PostList *
 InMemoryAllDocsPostList::next(Xapian::weight /*w_min*/)
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(!at_end());
     do {
        ++did;
@@ -320,6 +353,7 @@ InMemoryAllDocsPostList::next(Xapian::weight /*w_min*/)
 PostList *
 InMemoryAllDocsPostList::skip_to(Xapian::docid did_, Xapian::weight /*w_min*/)
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(!at_end());
     if (did <= did_) {
 	did = did_;
@@ -333,13 +367,14 @@ InMemoryAllDocsPostList::skip_to(Xapian::docid did_, Xapian::weight /*w_min*/)
 bool
 InMemoryAllDocsPostList::at_end() const
 {
+    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     return (did > db->termlists.size());
 }
 
 string
 InMemoryAllDocsPostList::get_description() const
 {
-    return "InMemoryAllDocsPostList " + om_tostring(did);
+    return "InMemoryAllDocsPostList " + str(did);
 }
 
 ///////////////////////////
@@ -347,10 +382,14 @@ InMemoryAllDocsPostList::get_description() const
 ///////////////////////////
 
 InMemoryDatabase::InMemoryDatabase()
-	: totdocs(0), totlen(0), positions_present(false)
+	: totdocs(0), totlen(0), positions_present(false), closed(false)
 {
     // Updates are applied immediately so we can't support transactions.
     transaction_state = TRANSACTION_UNIMPLEMENTED;
+
+    // We keep an empty entry in postlists for convenience of implementing
+    // allterms iteration and returning a PostList for an absent term.
+    postlists.insert(make_pair(string(), InMemoryTerm()));
 }
 
 InMemoryDatabase::~InMemoryDatabase()
@@ -358,32 +397,55 @@ InMemoryDatabase::~InMemoryDatabase()
     dtor_called();
 }
 
+void
+InMemoryDatabase::reopen()
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+}
+
+void
+InMemoryDatabase::close()
+{
+    // Free all the resources, and mark the db as closed.
+    postlists.clear();
+    termlists.clear();
+    doclists.clear();
+    valuelists.clear();
+    valuestats.clear();
+    doclengths.clear();
+    metadata.clear();
+    closed = true;
+}
+
 LeafPostList *
 InMemoryDatabase::open_post_list(const string & tname) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (tname.empty()) {
 	Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
 	return new InMemoryAllDocsPostList(ptrtothis);
     }
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
-    if (i == postlists.end() || i->second.term_freq == 0)
-	return new EmptyPostList();
-
+    if (i == postlists.end() || i->second.term_freq == 0) {
+	i = postlists.begin();
+	// Check that our dummy entry for string() is present.
+	Assert(i->first.empty());
+    }
     Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
-    LeafPostList * pl = new InMemoryPostList(ptrtothis, i->second);
-    Assert(!pl->at_end());
-    return pl;
+    return new InMemoryPostList(ptrtothis, i->second, tname);
 }
 
 bool
 InMemoryDatabase::doc_exists(Xapian::docid did) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     return (did > 0 && did <= termlists.size() && termlists[did - 1].is_valid);
 }
 
 Xapian::doccount
 InMemoryDatabase::get_termfreq(const string & tname) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
     if (i == postlists.end()) return 0;
     return i->second.term_freq;
@@ -392,35 +454,73 @@ InMemoryDatabase::get_termfreq(const string & tname) const
 Xapian::termcount
 InMemoryDatabase::get_collection_freq(const string &tname) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
     if (i == postlists.end()) return 0;
     return i->second.collection_freq;
 }
 
+Xapian::doccount
+InMemoryDatabase::get_value_freq(Xapian::valueno slot) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    if (i == valuestats.end()) return 0;
+    return i->second.freq;
+}
+
+std::string
+InMemoryDatabase::get_value_lower_bound(Xapian::valueno slot) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    if (i == valuestats.end()) return string();
+    return i->second.lower_bound;
+}
+
+std::string
+InMemoryDatabase::get_value_upper_bound(Xapian::valueno slot) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    if (i == valuestats.end()) return string();
+    return i->second.upper_bound;
+}
+
 Xapian::doccount
 InMemoryDatabase::get_doccount() const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     return totdocs;
 }
 
 Xapian::docid
 InMemoryDatabase::get_lastdocid() const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     return termlists.size();
 }
 
+totlen_t
+InMemoryDatabase::get_total_length() const
+{
+    return totlen;
+}
+
 Xapian::doclength
 InMemoryDatabase::get_avlength() const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (totdocs == 0) return 0;
     return Xapian::doclength(totlen) / totdocs;
 }
 
-Xapian::doclength
+Xapian::termcount
 InMemoryDatabase::get_doclength(Xapian::docid did) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (!doc_exists(did)) {
-	throw Xapian::DocNotFoundError(string("Docid ") + om_tostring(did) +
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
 				 string(" not found"));
     }
     return doclengths[did - 1];
@@ -429,10 +529,11 @@ InMemoryDatabase::get_doclength(Xapian::docid did) const
 TermList *
 InMemoryDatabase::open_term_list(Xapian::docid did) const
 {
-    if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 invalid");
+    if (closed) InMemoryDatabase::throw_database_closed();
+    Assert(did != 0);
     if (!doc_exists(did)) {
 	// FIXME: the docid in this message will be local, not global
-	throw Xapian::DocNotFoundError(string("Docid ") + om_tostring(did) +
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
 				 string(" not found"));
     }
     return new InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this), did,
@@ -440,22 +541,23 @@ InMemoryDatabase::open_term_list(Xapian::docid did) const
 }
 
 Xapian::Document::Internal *
-InMemoryDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
+InMemoryDatabase::open_document(Xapian::docid did, bool lazy) const
 {
-    // we're never lazy so ignore that flag
-    if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 invalid");
+    if (closed) InMemoryDatabase::throw_database_closed();
+    Assert(did != 0);
     if (!doc_exists(did)) {
+	if (lazy) return NULL;
 	// FIXME: the docid in this message will be local, not global
-	throw Xapian::DocNotFoundError(string("Docid ") + om_tostring(did) +
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
 				 string(" not found"));
     }
-    return new InMemoryDocument(this, did, doclists[did - 1],
-				valuelists[did - 1]);
+    return new InMemoryDocument(this, did);
 }
 
 std::string
 InMemoryDatabase::get_metadata(const std::string & key) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     map<string, string>::const_iterator i = metadata.find(key);
     if (i == metadata.end())
 	return string();
@@ -474,6 +576,7 @@ void
 InMemoryDatabase::set_metadata(const std::string & key,
 			       const std::string & value)
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (!value.empty()) {
 	metadata[key] = value;
     } else {
@@ -485,6 +588,7 @@ Xapian::termcount
 InMemoryDatabase::positionlist_count(Xapian::docid did,
 				     const string & tname) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (!doc_exists(did)) {
 	return 0;
     }
@@ -503,38 +607,61 @@ PositionList *
 InMemoryDatabase::open_position_list(Xapian::docid did,
 				     const string & tname) const
 {
-    if (!doc_exists(did)) {
-	throw Xapian::DocNotFoundError("Document id " + om_tostring(did) +
-				 " doesn't exist in inmemory database");
-    }
-    const InMemoryDoc &doc = termlists[did-1];
+    if (closed) InMemoryDatabase::throw_database_closed();
+    if (usual(doc_exists(did))) {
+	const InMemoryDoc &doc = termlists[did-1];
 
-    vector<InMemoryTermEntry>::const_iterator i;
-    for (i = doc.terms.begin(); i != doc.terms.end(); ++i) {
-	if (i->tname == tname) {
-	    return new InMemoryPositionList(i->positions);
+	vector<InMemoryTermEntry>::const_iterator i;
+	for (i = doc.terms.begin(); i != doc.terms.end(); ++i) {
+	    if (i->tname == tname) {
+		return new InMemoryPositionList(i->positions);
+	    }
 	}
     }
-    throw Xapian::RangeError("No positionlist for term in document.");
+    return new InMemoryPositionList(false);
 }
 
 void
 InMemoryDatabase::add_values(Xapian::docid did,
 			     const map<Xapian::valueno, string> &values_)
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (did > valuelists.size()) {
 	valuelists.resize(did);
     }
     valuelists[did-1] = values_;
+
+    // Update the statistics.
+    map<Xapian::valueno, string>::const_iterator j;
+    for (j = values_.begin(); j != values_.end(); ++j) {
+	std::pair<map<Xapian::valueno, ValueStats>::iterator, bool> i;
+	i = valuestats.insert(make_pair(j->first, ValueStats()));
+
+	// Now, modify the stored statistics.
+	if ((i.first->second.freq)++ == 0) {
+	    // If the value count was previously zero, set the upper and lower
+	    // bounds to the newly added value.
+	    i.first->second.lower_bound = j->second;
+	    i.first->second.upper_bound = j->second;
+	} else {
+	    // Otherwise, simply make sure they reflect the new value.
+	    if (j->second < i.first->second.lower_bound) {
+		i.first->second.lower_bound = j->second;
+	    }
+	    if (j->second > i.first->second.upper_bound) {
+		i.first->second.upper_bound = j->second;
+	    }
+	}
+    }
 }
 
-// We implicitly flush each modification right away, so nothing to do here.
+// We implicitly commit each modification right away, so nothing to do here.
 void
-InMemoryDatabase::flush()
+InMemoryDatabase::commit()
 {
 }
 
-// We implicitly flush each modification right away, so nothing to do here.
+// We implicitly commit each modification right away, so nothing to do here.
 void
 InMemoryDatabase::cancel()
 {
@@ -543,13 +670,24 @@ InMemoryDatabase::cancel()
 void
 InMemoryDatabase::delete_document(Xapian::docid did)
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     if (!doc_exists(did)) {
-	throw Xapian::DocNotFoundError(string("Docid ") + om_tostring(did) +
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
 				 string(" not found"));
     }
     termlists[did-1].is_valid = false;
     doclists[did-1] = "";
+    map<Xapian::valueno, string>::const_iterator j;
+    for (j = valuelists[did-1].begin(); j != valuelists[did-1].end(); ++j) {
+	map<Xapian::valueno, ValueStats>::iterator i;
+	i = valuestats.find(j->first);
+	if (--(i->second.freq) == 0) {
+	    i->second.lower_bound.resize(0);
+	    i->second.upper_bound.resize(0);
+	}
+    }
     valuelists[did-1].clear();
+
     totlen -= doclengths[did-1];
     doclengths[did-1] = 0;
     totdocs--;
@@ -581,12 +719,21 @@ void
 InMemoryDatabase::replace_document(Xapian::docid did,
 				   const Xapian::Document & document)
 {
-    DEBUGLINE(DB, "InMemoryDatabase::replace_document(): replacing doc "
-	          << did);
+    LOGCALL_VOID(DB, "InMemoryDatabase::replace_document", did | document);
+
+    if (closed) InMemoryDatabase::throw_database_closed();
 
     if (doc_exists(did)) { 
-	doclists[did - 1] = "";
-	valuelists[did - 1].clear();
+	map<Xapian::valueno, string>::const_iterator j;
+	for (j = valuelists[did-1].begin(); j != valuelists[did-1].end(); ++j) {
+	    map<Xapian::valueno, ValueStats>::iterator i;
+	    i = valuestats.find(j->first);
+	    if (--(i->second.freq) == 0) {
+		i->second.lower_bound.resize(0);
+		i->second.upper_bound.resize(0);
+	    }
+	}
+
 	totlen -= doclengths[did - 1];
 	totdocs--;
     } else if (did > termlists.size()) {
@@ -626,13 +773,14 @@ InMemoryDatabase::replace_document(Xapian::docid did,
 Xapian::docid
 InMemoryDatabase::add_document(const Xapian::Document & document)
 {
-    Xapian::docid did = make_doc(document.get_data());
+    LOGCALL(DB, Xapian::docid, "InMemoryDatabase::add_document", document);
+    if (closed) InMemoryDatabase::throw_database_closed();
 
-    DEBUGLINE(DB, "InMemoryDatabase::add_document(): adding doc " << did);
+    Xapian::docid did = make_doc(document.get_data());
 
     finish_add_doc(did, document);
 
-    return did;
+    RETURN(did);
 }
 
 void
@@ -641,11 +789,10 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
     {
 	map<Xapian::valueno, string> values;
 	Xapian::ValueIterator k = document.values_begin();
-	Xapian::ValueIterator k_end = document.values_end();
-	for ( ; k != k_end; ++k) {
+	for ( ; k != document.values_end(); ++k) {
 	    values.insert(make_pair(k.get_valueno(), *k));
-	    DEBUGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding value "
-		      << k.get_valueno() << " -> " << *k);
+	    LOGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding value " <<
+			k.get_valueno() << " -> " << *k);
 	}
 	add_values(did, values);
     }
@@ -656,8 +803,7 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
     for ( ; i != i_end; ++i) {
 	make_term(*i);
 
-	DEBUGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding term "
-		  << *i);
+	LOGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding term " << *i);
 	Xapian::PositionIterator j = i.positionlist_begin();
 	Xapian::PositionIterator j_end = i.positionlist_end();
 
@@ -740,6 +886,7 @@ void InMemoryDatabase::make_posting(InMemoryDoc * doc,
 bool
 InMemoryDatabase::term_exists(const string & tname) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     Assert(!tname.empty());
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
     if (i == postlists.end()) return false;
@@ -749,13 +896,21 @@ InMemoryDatabase::term_exists(const string & tname) const
 bool
 InMemoryDatabase::has_positions() const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     return positions_present;
 }
 
 TermList *
 InMemoryDatabase::open_allterms(const string & prefix) const
 {
+    if (closed) InMemoryDatabase::throw_database_closed();
     return new InMemoryAllTermsList(&postlists,
 				    Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this),
 				    prefix);
 }
+
+void
+InMemoryDatabase::throw_database_closed()
+{
+    throw Xapian::DatabaseError("Database has been closed");
+}
