@@ -1,9 +1,9 @@
-/* inmemory_database.h: C++ class definition for multiple database access
+/* inmemory_database.h: C++ class definition for inmemory database access
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
- * Copyright 2006 Richard Boulton
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,10 +32,14 @@
 #include <algorithm>
 #include <xapian/document.h>
 #include "inmemory_positionlist.h"
-#include <omassert.h>
+#include "internaltypes.h"
+#include "omassert.h"
+#include "noreturn.h"
 
 using namespace std;
 
+class ValueStats;
+
 // Class representing a posting (a term/doc pair, and
 // all the relevant positional information, is a single posting)
 class InMemoryPosting {
@@ -144,13 +148,13 @@ class InMemoryPostList : public LeafPostList {
 	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
 
 	InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db,
-			 const InMemoryTerm & term);
+			 const InMemoryTerm & imterm, const std::string & term_);
     public:
 	Xapian::doccount get_termfreq() const;
 
 	Xapian::docid       get_docid() const;     // Gets current docid
-	Xapian::doclength   get_doclength() const; // Length of current document
-        Xapian::termcount   get_wdf() const;	      // Within Document Frequency
+	Xapian::termcount   get_doclength() const; // Length of current document
+	Xapian::termcount   get_wdf() const;	   // Within Document Frequency
 	PositionList * read_position_list();
 	PositionList * open_position_list() const;
 
@@ -178,7 +182,7 @@ class InMemoryAllDocsPostList : public LeafPostList {
 	Xapian::doccount get_termfreq() const;
 
 	Xapian::docid       get_docid() const;     // Gets current docid
-	Xapian::doclength   get_doclength() const; // Length of current document
+	Xapian::termcount   get_doclength() const; // Length of current document
 	Xapian::termcount   get_wdf() const;       // Within Document Frequency
 	PositionList * read_position_list();
 	PositionList * open_position_list() const;
@@ -204,11 +208,12 @@ class InMemoryTermList : public TermList {
 
 	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
 	Xapian::docid did;
-	Xapian::doclength document_length;
+	Xapian::termcount document_length;
 
-	InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db, Xapian::docid did,
+	InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db,
+			 Xapian::docid did,
 			 const InMemoryDoc & doc,
-			 Xapian::doclength len);
+			 Xapian::termcount len);
     public:
 	Xapian::termcount get_approx_size() const;
 
@@ -219,33 +224,41 @@ class InMemoryTermList : public TermList {
 	Xapian::termcount get_wdf() const; // Number of occurrences of term in current doc
 	Xapian::doccount get_termfreq() const;  // Number of docs indexed by term
 	TermList * next();
+	TermList * skip_to(const std::string & term);
 	bool at_end() const;
 	Xapian::termcount positionlist_count() const;
 	Xapian::PositionIterator positionlist_begin() const;
 };
 
+class InMemoryDocument;
+
 /** A database held entirely in memory.
  *
  *  This is a prototype database, mainly used for debugging and testing.
  */
 class InMemoryDatabase : public Xapian::Database::Internal {
     friend class InMemoryAllDocsPostList;
+    friend class InMemoryDocument;
 
     map<string, InMemoryTerm> postlists;
     vector<InMemoryDoc> termlists;
     vector<std::string> doclists;
     vector<std::map<Xapian::valueno, string> > valuelists;
+    std::map<Xapian::valueno, ValueStats> valuestats;
 
-    vector<Xapian::doclength> doclengths;
+    vector<Xapian::termcount> doclengths;
 
     std::map<string, string> metadata;
 
     Xapian::doccount totdocs;
 
-    Xapian::doclength totlen;
+    totlen_t totlen;
 
     bool positions_present;
 
+    // Flag, true if the db has been closed.
+    bool closed;
+
     // Stop copy / assignment being allowed
     InMemoryDatabase& operator=(const InMemoryDatabase &);
     InMemoryDatabase(const InMemoryDatabase &);
@@ -269,7 +282,7 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     //@{
     /** Implementation of virtual methods: see Database for details.
      */
-    void flush();
+    void commit();
     void cancel();
 
     Xapian::docid add_document(const Xapian::Document & document);
@@ -278,7 +291,7 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     // a problem as we only try to call them through the base class
     // (where they aren't hidden) but some compilers generate a warning
     // about the hiding.
-#if (!defined __GNUC__ && !defined _MSC_VER) || __GNUC__ > 2
+#ifndef _MSC_VER
     using Xapian::Database::Internal::delete_document;
     using Xapian::Database::Internal::replace_document;
 #endif
@@ -295,23 +308,32 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 
     ~InMemoryDatabase();
 
+    void reopen();
+    void close();
+    bool is_closed() const { return closed; }
+
     Xapian::doccount get_doccount() const;
 
     Xapian::docid get_lastdocid() const;
 
+    totlen_t get_total_length() const;
     Xapian::doclength get_avlength() const;
-    Xapian::doclength get_doclength(Xapian::docid did) const;
+    Xapian::termcount get_doclength(Xapian::docid did) const;
 
     Xapian::doccount get_termfreq(const string & tname) const;
     Xapian::termcount get_collection_freq(const string & tname) const;
+    Xapian::doccount get_value_freq(Xapian::valueno slot) const;
+    std::string get_value_lower_bound(Xapian::valueno slot) const;
+    std::string get_value_upper_bound(Xapian::valueno slot) const;
     bool term_exists(const string & tname) const;
     bool has_positions() const;
 
     LeafPostList * open_post_list(const string & tname) const;
     TermList * open_term_list(Xapian::docid did) const;
-    Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
+    Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy) const;
 
     std::string get_metadata(const std::string & key) const;
+    TermList * open_metadata_keylist(const std::string &prefix) const;
     void set_metadata(const std::string & key, const std::string & value);
 
     Xapian::termcount positionlist_count(Xapian::docid did,
@@ -319,6 +341,8 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     PositionList * open_position_list(Xapian::docid did,
 				      const string & tname) const;
     TermList * open_allterms(const string & prefix) const;
+
+    XAPIAN_NORETURN(static void throw_database_closed());
 };
 
 #endif /* OM_HGUARD_INMEMORY_DATABASE_H */
