@@ -1,7 +1,7 @@
 /* multi_postlist.cc: interface to multiple database access
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,14 +20,14 @@
  */
 
 #include <config.h>
-#include <stdio.h>
 
-#include "omassert.h"
-#include "omdebug.h"
 #include "multi_postlist.h"
+
+#include "debuglog.h"
+#include "omassert.h"
 #include "utils.h"
 
-#ifdef XAPIAN_DEBUG_PARANOID
+#ifdef XAPIAN_ASSERTIONS_PARANOID
 #include "xapian/database.h"
 #endif
 
@@ -52,29 +52,61 @@ MultiPostList::~MultiPostList()
 }
 
 Xapian::doccount
-MultiPostList::get_termfreq() const
+MultiPostList::get_termfreq_min() const
 {
     // Should never get called.
     Assert(false);
     return 0;
 }
 
+Xapian::doccount
+MultiPostList::get_termfreq_max() const
+{
+    return MultiPostList::get_termfreq_min();
+}
+
+Xapian::doccount
+MultiPostList::get_termfreq_est() const
+{
+    return MultiPostList::get_termfreq_min();
+}
+
+Xapian::weight
+MultiPostList::get_maxweight() const
+{
+    return MultiPostList::get_weight();
+}
+
+Xapian::weight
+MultiPostList::get_weight() const
+{
+    // Should never get called.
+    Assert(false);
+    return 0;
+}
+
+Xapian::weight
+MultiPostList::recalc_maxweight()
+{
+    return MultiPostList::get_weight();
+}
+
 Xapian::docid
 MultiPostList::get_docid() const
 {
-    DEBUGCALL(DB, Xapian::docid, "MultiPostList::get_docid", "");
+    LOGCALL(DB, Xapian::docid, "MultiPostList::get_docid", NO_ARGS);
     Assert(!at_end());
     Assert(currdoc != 0);
     RETURN(currdoc);
 }
 
-Xapian::doclength
+Xapian::termcount
 MultiPostList::get_doclength() const
 {
-    DEBUGCALL(DB, Xapian::doclength, "MultiPostList::get_doclength", "");
+    LOGCALL(DB, Xapian::termcount, "MultiPostList::get_doclength", NO_ARGS);
     Assert(!at_end());
     Assert(currdoc != 0);
-    Xapian::doclength result = postlists[(currdoc - 1) % multiplier]->get_doclength();
+    Xapian::termcount result = postlists[(currdoc - 1) % multiplier]->get_doclength();
     AssertEqParanoid(result, this_db.get_doclength(get_docid()));
     RETURN(result);
 }
@@ -94,7 +126,7 @@ MultiPostList::open_position_list() const
 PostList *
 MultiPostList::next(Xapian::weight w_min)
 {
-    DEBUGCALL(DB, PostList *, "MultiPostList::next", w_min);
+    LOGCALL(DB, PostList *, "MultiPostList::next", w_min);
     Assert(!at_end());
 
     Xapian::docid newdoc = 0;
@@ -117,12 +149,12 @@ MultiPostList::next(Xapian::weight w_min)
 	offset++;
     }
     if (newdoc) {
-	DEBUGLINE(DB, "MultiPostList::next() newdoc=" << newdoc <<
-		  " (olddoc=" << currdoc << ")");
+	LOGLINE(DB, "MultiPostList::next() newdoc=" << newdoc <<
+		    " (olddoc=" << currdoc << ")");
 	currdoc = newdoc;
     } else {
-	DEBUGLINE(DB, "MultiPostList::next() finished" <<
-		  " (olddoc=" << currdoc << ")");
+	LOGLINE(DB, "MultiPostList::next() finished" <<
+		    " (olddoc=" << currdoc << ")");
 	finished = true;
     }
     RETURN(NULL);
@@ -131,7 +163,7 @@ MultiPostList::next(Xapian::weight w_min)
 PostList *
 MultiPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 {
-    DEBUGCALL(DB, PostList *, "MultiPostList::skip_to", did << ", " << w_min);
+    LOGCALL(DB, PostList *, "MultiPostList::skip_to", did | w_min);
     Assert(!at_end());
     Xapian::docid newdoc = 0;
     Xapian::docid offset = 0;
