@@ -1,7 +1,7 @@
 /* net_termlist.cc
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2006,2007 Olly Betts
+ * Copyright 2002,2003,2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,17 +20,14 @@
  */
 
 #include <config.h>
-#include <stdio.h>
 
-#include "omassert.h"
-#include "omdebug.h"
 #include "net_termlist.h"
 
-#include <vector>
+#include "omassert.h"
 
 #include <xapian/error.h>
 
-NetworkTermList::NetworkTermList(Xapian::doclength document_length_,
+NetworkTermList::NetworkTermList(Xapian::termcount document_length_,
 				 Xapian::doccount database_size_,
 				 Xapian::Internal::RefCntPtr<const RemoteDatabase> this_db_,
 				 Xapian::docid did_)
@@ -98,6 +95,16 @@ NetworkTermList::next()
     return NULL;
 }
 
+TermList *
+NetworkTermList::skip_to(const string & term)
+{
+    while (current_position != items.end() && current_position->tname < term) {
+	++current_position;
+    }
+    started = true;
+    return NULL;
+}
+
 bool
 NetworkTermList::at_end() const
 {
