@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -26,13 +26,15 @@
 
 #include <string>
 
+#include "internaltypes.h"
+
 #include <xapian/base.h>
 #include <xapian/types.h>
 #include <xapian/database.h>
 #include <xapian/document.h>
 #include <xapian/positioniterator.h>
 #include <xapian/termiterator.h>
-#include "omdebug.h"
+#include <xapian/valueiterator.h>
 
 using namespace std;
 
@@ -41,9 +43,15 @@ class RemoteDatabase;
 
 typedef Xapian::TermIterator::Internal TermList;
 typedef Xapian::PositionIterator::Internal PositionList;
+typedef Xapian::ValueIterator::Internal ValueList;
+
+// Used by flint and chert.
+const int XAPIAN_DB_READONLY = 0;
 
 namespace Xapian {
 
+struct ReplicationInfo;
+
 /** Base class for databases.
  */
 class Database::Internal : public Xapian::Internal::RefCntBase {
@@ -65,12 +73,11 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 
 	bool transaction_active() const { return int(transaction_state) > 0; }
 
-	/** Create a database - called only by derived classes.
-	 */
-	Internal();
+	/** Create a database - called only by derived classes. */
+	Internal() : transaction_state(TRANSACTION_NONE) { }
 
 	/** Internal method to perform cleanup when a writable database is
-	 *  destroyed with unflushed changes.
+	 *  destroyed with uncommitted changes.
 	 *
 	 *  A derived class' destructor should call this method before
 	 *  destroying the database to ensure that no sessions or
@@ -112,6 +119,9 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	virtual Xapian::docid get_lastdocid() const = 0;
 
+	/** Return the total length of all documents in this database. */
+	virtual totlen_t get_total_length() const = 0;
+
 	/** Return the average length of a document in this (sub) database.
 	 *
 	 *  See Database::Internal::get_doclength() for the meaning of document
@@ -129,7 +139,7 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 *  @param did  The document id of the document whose length is
 	 *              being requested.
 	 */
-	virtual Xapian::doclength get_doclength(Xapian::docid did) const = 0;
+	virtual Xapian::termcount get_doclength(Xapian::docid did) const = 0;
 
 	/** Return the number of documents indexed by a given term.  This
 	 *  may be an approximation, but must be an upper bound (ie,
@@ -150,6 +160,45 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	virtual Xapian::termcount get_collection_freq(const string & tname) const = 0;
 
+	/** Return the frequency of a given value slot.
+	 *
+	 *  This is the number of documents which have a (non-empty) value
+	 *  stored in the slot.
+	 *
+	 *  @param valno The value slot to examine.
+	 *
+	 *  @exception UnimplementedError The frequency of the value isn't
+	 *  available for this database type.
+	 */
+	virtual Xapian::doccount get_value_freq(Xapian::valueno valno) const;
+
+	/** Get a lower bound on the values stored in the given value slot.
+	 *
+	 *  If the lower bound isn't available for the given database type,
+	 *  this will return the lowest possible bound - the empty string.
+	 *
+	 *  @param valno The value slot to examine.
+	 */
+	virtual std::string get_value_lower_bound(Xapian::valueno valno) const;
+
+	/** Get an upper bound on the values stored in the given value slot.
+	 *
+	 *  @param valno The value slot to examine.
+	 *
+	 *  @exception UnimplementedError The upper bound of the values isn't
+	 *  available for this database type.
+	 */
+	virtual std::string get_value_upper_bound(Xapian::valueno valno) const;
+
+	/// Get a lower bound on the length of a document in this DB.
+	virtual Xapian::termcount get_doclength_lower_bound() const;
+
+	/// Get an upper bound on the length of a document in this DB.
+	virtual Xapian::termcount get_doclength_upper_bound() const;
+
+	/// Get an upper bound on the wdf of term @a term.
+	virtual Xapian::termcount get_wdf_upper_bound(const std::string & term) const;
+
 	/** Check whether a given term is in the database.
 	 *
 	 *  @param tname  The term whose presence is being checked.
@@ -172,15 +221,25 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 *  @param tname  The term whose posting list is being requested.
 	 *
 	 *  @return       A pointer to the newly created posting list.
-	 *		  If the term doesn't exist, a new EmptyPostList
-	 *		  object (or an object which behaves the same way) is
-	 *		  returned instead, which makes it easier
-	 *		  to implement a search over multiple databases.
-	 *                This object must be deleted by the caller after
+	 *		  If the term doesn't exist, a LeafPostList object
+	 *		  returning no documents is returned, which makes it
+	 *		  easier to implement a search over multiple databases.
+	 *		  This object must be deleted by the caller after
 	 *                use.
 	 */
 	virtual LeafPostList * open_post_list(const string & tname) const = 0;
 
+	/** Open a value stream.
+	 *
+	 *  This returns the value in a particular slot for each document.
+	 *
+	 *  @param slot	The value slot.
+	 *
+	 *  @return	Pointer to a new ValueList object which should be
+	 *		deleted by the caller once it is no longer needed.
+	 */
+	virtual ValueList * open_value_list(Xapian::valueno slot) const;
+
 	/** Open a term list.
 	 *
 	 *  This is a list of all the terms contained by a given document.
@@ -336,15 +395,19 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	virtual void reopen();
 
+	/** Close the database
+	 */
+	virtual void close() = 0;
+
 	//////////////////////////////////////////////////////////////////
 	// Modifying the database:
 	// =======================
 
-	/** Flush pending modifications to the database.
+	/** Commit pending modifications to the database.
 	 *
-	 *  See WritableDatabase::flush() for more information.
+	 *  See WritableDatabase::commit() for more information.
 	 */
-	virtual void flush();
+	virtual void commit();
 
 	/** Cancel pending modifications to the database. */
 	virtual void cancel();
@@ -414,6 +477,31 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	virtual Xapian::Document::Internal * collect_document(Xapian::docid did) const;
 	//@}
 
+	/** Write a set of changesets to a file descriptor.
+	 *
+	 *  This call may reopen the database, leaving it pointing to a more
+	 *  recent version of the database.
+	 */
+	virtual void write_changesets_to_fd(int fd,
+					    const std::string & start_revision,
+					    bool need_whole_db,
+					    Xapian::ReplicationInfo * info);
+
+	/// Get a string describing the current revision of the database.
+	virtual string get_revision_info() const;
+
+	/** Get a UUID for the database.
+	 *
+	 *  The UUID will persist for the lifetime of the database.
+	 *
+	 *  Replicas (eg, made with the replication protocol, or by copying all
+	 *  the database files) will have the same UUID.  However, copies (made
+	 *  with copydatabase, or xapian-compact) will have different UUIDs.
+	 *
+	 *  If the backend does not support UUIDs the empty string is returned.
+	 */
+	virtual string get_uuid() const;
+
 	/** Notify the database that document is no longer valid.
 	 *
 	 *  This is used to invalidate references to a document kept by a
