@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2007 Olly Betts
+ * Copyright 2003,2004,2005,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,9 +33,13 @@
 
 using namespace std;
 
+class DocumentValueList;
+class ValueStreamDocument;
+
 /// A document in the database, possibly plus modifications.
 class Xapian::Document::Internal : public Xapian::Internal::RefCntBase {
-    friend class Xapian::ValueIterator;
+    friend class ::DocumentValueList;
+    friend class ::ValueStreamDocument;
     public:
 	/// Type to store values in.
 	typedef map<Xapian::valueno, string> document_values;
@@ -43,14 +47,15 @@ class Xapian::Document::Internal : public Xapian::Internal::RefCntBase {
 	/// Type to store terms in.
 	typedef map<string, OmDocumentTerm> document_terms;
 
+    protected:
+	/// The database this document is in.
+	Xapian::Internal::RefCntPtr<const Xapian::Database::Internal> database;
+
     private:
         // Prevent copying
         Internal(const Internal &);
         Internal & operator=(const Internal &);
 
-	/// The database this document is in.
-	Xapian::Internal::RefCntPtr<const Xapian::Database::Internal> database;
-
 	bool data_here;
 	mutable bool values_here; // FIXME mutable is a hack
 	mutable bool terms_here;
@@ -61,9 +66,6 @@ class Xapian::Document::Internal : public Xapian::Internal::RefCntBase {
 	/// The values associated with this document.
 	mutable document_values values; // FIXME mutable is a hack
 
-	/// The value numbers, for use by ValueIterators.
-	mutable vector<Xapian::valueno> value_nos; // FIXME mutable is a hack
-
 	/// The terms (and their frequencies and positions) in this document.
 	mutable document_terms terms;
 
@@ -77,12 +79,11 @@ class Xapian::Document::Internal : public Xapian::Internal::RefCntBase {
 
     private:
 	// Functions for backend to implement
-	virtual string do_get_value(Xapian::valueno /*valueno*/) const { return ""; }
-	virtual map<Xapian::valueno, string> do_get_all_values() const {
-	    map<Xapian::valueno, string> none;
-	    return none;
+	virtual string do_get_value(Xapian::valueno /*valueno*/) const { return string(); }
+	virtual void do_get_all_values(map<Xapian::valueno, string> & values_) const {
+	    values_.clear();
 	}
-	virtual string do_get_data() const { return ""; }
+	virtual string do_get_data() const { return string(); }
 
     public:
 	/** Get value by value number.
@@ -103,6 +104,17 @@ class Xapian::Document::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	string get_value(Xapian::valueno valueid) const;
 
+	/** Set all the values.
+	 *
+	 *  @param values_	The values to set - passed by non-const reference, and
+	 *			may be modified by the call.
+	 */
+	void set_all_values(map<Xapian::valueno, string> & values_) {
+	    // For efficiency we just swap the old and new value maps.
+	    swap(values, values_);
+	    values_here = true;
+	}
+
 	Xapian::valueno values_count() const;
 	void add_value(Xapian::valueno, const string &);
 	void remove_value(Xapian::valueno);
