@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,20 +24,20 @@
 #ifndef OM_HGUARD_OMENQUIREINTERNAL_H
 #define OM_HGUARD_OMENQUIREINTERNAL_H
 
-#include <xapian/database.h>
-#include <xapian/document.h>
-#include <xapian/enquire.h>
-#include <xapian/sorter.h>
-#include <xapian/query.h>
+#include "xapian/database.h"
+#include "xapian/document.h"
+#include "xapian/enquire.h"
+#include "xapian/query.h"
+#include "xapian/keymaker.h"
+
 #include <algorithm>
-#include <math.h>
+#include <cmath>
 #include <map>
 #include <set>
 
 using namespace std;
 
 class OmExpand;
-class RSetI;
 class MultiMatch;
 
 namespace Xapian {
@@ -46,28 +47,6 @@ class TermIterator;
 
 namespace Internal {
 
-/** An item in the ESet.
- *  This item contains the termname, and the weight calculated for
- *  the document.
- */
-class ESetItem {
-    public:
-	ESetItem(Xapian::weight wt_, string tname_) : wt(wt_), tname(tname_) { }
-
-	void swap(ESetItem & o) {
-	    std::swap(wt, o.wt);
-	    std::swap(tname, o.tname);
-	}
-
-	/// Weight calculated.
-	Xapian::weight wt;
-	/// Term suggested.
-	string tname;
-
-	/// Return a string describing this object.
-	string get_description() const;
-};
-
 /** An item resulting from a query.
  *  This item contains the document id, and the weight calculated for
  *  the document.
@@ -155,6 +134,8 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 
 	Xapian::valueno collapse_key;
 
+	Xapian::doccount collapse_max;
+
 	Xapian::Enquire::docid_order order;
 
 	percent percent_cutoff;
@@ -165,7 +146,7 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 	sort_setting sort_by;
 	bool sort_value_forward;
 
-	Sorter * sorter;
+	KeyMaker * sorter;
 
 	/** The error handler, if set.  (0 if not set).
 	 */
@@ -178,6 +159,8 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	mutable Weight * weight;
 
+	vector<MatchSpy *> spies;
+
 	Internal(const Xapian::Database &databases, ErrorHandler * errorhandler_);
 	~Internal();
 
@@ -193,14 +176,17 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 	const Query & get_query();
 	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		      Xapian::doccount check_at_least,
-		      const RSet *omrset, const MatchDecider *mdecider,
-		      const MatchDecider *matchspy) const;
+		      const RSet *omrset,
+		      const MatchDecider *mdecider,
+		      const MatchDecider *matchspy_legacy) const;
 	ESet get_eset(Xapian::termcount maxitems, const RSet & omrset, int flags,
 		      double k, const ExpandDecider *edecider) const;
 
 	TermIterator get_matching_terms(Xapian::docid did) const;
 	TermIterator get_matching_terms(const Xapian::MSetIterator &it) const;
 
+	Xapian::doccount get_termfreq(const string &tname) const;
+
 	void register_match_decider(const string &name,
 				    const MatchDecider *mdecider);
 
@@ -238,6 +224,8 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	struct TermFreqAndWeight {
 	    TermFreqAndWeight() { }
+	    explicit TermFreqAndWeight(Xapian::doccount tf)
+		: termfreq(tf), termweight(0.0) { }
 	    TermFreqAndWeight(Xapian::doccount tf, Xapian::weight wt)
 		: termfreq(tf), termweight(wt) { }
 	    Xapian::doccount termfreq;
@@ -263,6 +251,12 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 
 	Xapian::doccount matches_upper_bound;
 
+	Xapian::doccount uncollapsed_lower_bound;
+
+	Xapian::doccount uncollapsed_estimated;
+
+	Xapian::doccount uncollapsed_upper_bound;
+
 	Xapian::weight max_possible;
 
 	Xapian::weight max_attained;
@@ -273,6 +267,9 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 		  matches_lower_bound(0),
 		  matches_estimated(0),
 		  matches_upper_bound(0),
+		  uncollapsed_lower_bound(0),
+		  uncollapsed_estimated(0),
+		  uncollapsed_upper_bound(0),
 		  max_possible(0),
 		  max_attained(0) {}
 
@@ -281,6 +278,9 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 	     Xapian::doccount matches_upper_bound_,
 	     Xapian::doccount matches_lower_bound_,
 	     Xapian::doccount matches_estimated_,
+	     Xapian::doccount uncollapsed_upper_bound_,
+	     Xapian::doccount uncollapsed_lower_bound_,
+	     Xapian::doccount uncollapsed_estimated_,
 	     Xapian::weight max_possible_,
 	     Xapian::weight max_attained_,
 	     vector<Xapian::Internal::MSetItem> &items_,
@@ -292,6 +292,9 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 		  matches_lower_bound(matches_lower_bound_),
 		  matches_estimated(matches_estimated_),
 		  matches_upper_bound(matches_upper_bound_),
+		  uncollapsed_lower_bound(uncollapsed_lower_bound_),
+		  uncollapsed_estimated(uncollapsed_estimated_),
+		  uncollapsed_upper_bound(uncollapsed_upper_bound_),
 		  max_possible(max_possible_),
 		  max_attained(max_attained_) {
 	    std::swap(items, items_);
@@ -311,27 +314,6 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 	void fetch_items(Xapian::doccount first, Xapian::doccount last) const;
 };
 
-class ESet::Internal : public Xapian::Internal::RefCntBase {
-    friend class ESet;
-    friend class ESetIterator;
-    friend class ::OmExpand;
-    private:
-	/// A list of items comprising the (selected part of the) ESet.
-	vector<Xapian::Internal::ESetItem> items;
-
-	/** A lower bound on the number of terms which are in the full
-	 *  set of results of the expand.  This will be greater than or
-	 *  equal to items.size()
-	 */
-	Xapian::termcount ebound;
-
-    public:
-	Internal() : ebound(0) {}
-
-	/// Return a string describing this object.
-	string get_description() const;
-};
-
 class RSet::Internal : public Xapian::Internal::RefCntBase {
     friend class Xapian::RSet;
 
