@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006,2007,2008,2010 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,8 +30,8 @@
 
 #include "gnu_getopt.h"
 
-#include <string.h>
-#include <stdlib.h>
+#include <cstring>
+#include <cstdlib>
 #include "safeerrno.h"
 
 using namespace Xapian;
@@ -56,7 +56,8 @@ static void show_usage() {
 "  -s, --stemmer=LANG    set the stemming language, the default is 'none'\n"
 "  -1                    output one list entry per line\n"
 "  -V                    output values for each document referred to\n"
-"  -V<valueno>           output value valueno for each document in the database\n"
+"  -V<valueno>           output value valueno for each document referred to\n"
+"                        (or each document in the database if no -r options)\n"
 "  -d                    output document data for each document referred to\n"
 "  -v                    extra info (wdf and len for postlist;\n"
 "                        wdf and termfreq for termlist; number of terms for db;\n"
@@ -69,8 +70,13 @@ static void
 show_db_stats(Database &db)
 {
     // Display a few database stats.
+    cout << "UUID = " << db.get_uuid() << endl;
     cout << "number of documents = " << db.get_doccount() << endl;
     cout << "average document length = " << db.get_avlength() << endl;
+    cout << "document length lower bound = " << db.get_doclength_lower_bound()
+	 << endl;
+    cout << "document length upper bound = " << db.get_doclength_upper_bound()
+	 << endl;
     cout << "highest document id ever used = " << db.get_lastdocid() << endl;
 
     if (verbose) {
@@ -93,8 +99,7 @@ show_values(Database &db, docid docid, char sep)
 {
     Document doc = db.get_document(docid);
     ValueIterator v = doc.values_begin();
-    ValueIterator vend = doc.values_end();
-    while (v != vend) {
+    while (v != doc.values_end()) {
 	cout << sep << v.get_valueno() << ':' << *v;
 	++v;
     }
@@ -113,6 +118,20 @@ show_values(Database &db,
     }
 }
 
+static void
+show_value(Database &db,
+	   vector<docid>::const_iterator i,
+	   vector<docid>::const_iterator end,
+	   Xapian::valueno slot)
+{
+    while (i != end) {
+	Xapian::docid did = *i;
+	cout << "Value " << slot << " for record #" << did << ": "
+	     << db.get_document(did).get_value(slot) << endl;
+	++i;
+    }
+}
+
 static void
 show_docdata(Database &db, docid docid, char sep)
 {
@@ -172,8 +191,7 @@ show_termlists(Database &db,
 static Stem stemmer;
 
 int
-main(int argc, char **argv)
-{
+main(int argc, char **argv) try {
     if (argc > 1 && argv[1][0] == '-') {
 	if (strcmp(argv[1], "--help") == 0) {
 	    cout << PROG_NAME" - "PROG_DESC"\n\n";
@@ -191,11 +209,11 @@ main(int argc, char **argv)
     vector<string> terms;
     vector<string> dbs;
 
-    valueno valno = 0; // Avoid "may be used uninitialised" warnings.
-    bool valno_set = false;
+    valueno slot = 0; // Avoid "may be used uninitialised" warnings.
+    bool slot_set = false;
 
     int c;
-    while ((c = gnu_getopt(argc, argv, "ar:t:s:1vkV::d")) != -1) {
+    while ((c = gnu_getopt(argc, argv, "ar:t:s:1vV::d")) != -1) {
 	switch (c) {
 	    case 'a':
 		all_terms = true;
@@ -225,8 +243,7 @@ main(int argc, char **argv)
 	    case '1':
 		separator = '\n';
 		break;
-	    case 'V': case 'k': /* -k for backward compatibility */
-		showvalues = true;
+	    case 'V':
 		if (optarg) {
 		    char * end;
 		    errno = 0;
@@ -235,12 +252,14 @@ main(int argc, char **argv)
 			cout << "Non-numeric value slot: " << optarg << endl;
 			exit(1);
 		    }
-		    valno = Xapian::valueno(n);
-		    if (errno == ERANGE || valno != n) {
+		    slot = Xapian::valueno(n);
+		    if (errno == ERANGE || slot != n) {
 			cout << "Value slot out of range: " << optarg << endl;
 			exit(1);
 		    }
-		    valno_set = true;
+		    slot_set = true;
+		} else {
+		    showvalues = true;
 		}
 		break;
 	    case 'd':
@@ -278,102 +297,95 @@ main(int argc, char **argv)
 	}
     }
 
-    try {
-	if (!all_terms && terms.empty() && recnos.empty() && !valno_set) {
-	    show_db_stats(db);
-	    return 0;
+    if (!all_terms && terms.empty() && recnos.empty() && !slot_set) {
+	// Show some statistics about the database.
+	show_db_stats(db);
+	return 0;
+    }
+
+    if (all_terms) {
+	show_termlist(db, 0);
+    }
+
+    if (!recnos.empty()) {
+	if (showvalues) {
+	    show_values(db, recnos.begin(), recnos.end());
+	} else if (slot_set) {
+	    show_value(db, recnos.begin(), recnos.end(), slot);
 	}
 
-	if (all_terms) {
-	    show_termlist(db, 0);
+	if (showdocdata) {
+	    show_docdata(db, recnos.begin(), recnos.end());
 	}
-
-	if (!recnos.empty()) {
-	    if (showvalues) {
-		show_values(db, recnos.begin(), recnos.end());
-	    }
-
-	    if (showdocdata) {
-		show_docdata(db, recnos.begin(), recnos.end());
-	    }
-	}
-
-	if (valno_set) {
-	    doccount n = db.get_doccount();
-	    docid did = 0;
-	    docid hwm = db.get_lastdocid();
-	    cout << "Value " << valno << " for each document:";
-	    while (n && did != hwm) {
-		try {
-		    Document doc = db.get_document(++did);
-		    string val = doc.get_value(valno);
-		    if (!val.empty())
-			cout << separator << did << ':' << doc.get_value(valno);
-		    --n;
-		} catch (DocNotFoundError &) {
-		}
+    } else {
+	if (slot_set) {
+	    cout << "Value " << slot << " for each document:";
+	    ValueIterator it = db.valuestream_begin(slot);
+	    while (it != db.valuestream_end(slot)) {
+		cout << separator << it.get_docid() << ':' << *it;
+		++it;
 	    }
 	    cout << endl;
 	}
-
-	if (terms.empty()) {
-	    show_termlists(db, recnos.begin(), recnos.end());
-	    return 0;
-	}
-
-	vector<string>::const_iterator i;
-	for (i = terms.begin(); i != terms.end(); i++) {
-	    string term = stemmer(*i);
-	    PostingIterator p = db.postlist_begin(term);
-	    PostingIterator pend = db.postlist_end(term);
-	    if (p == pend) {
-		cout << "term `" << term << "' not in database\n";
-		continue;
-	    }
-	    if (recnos.empty()) {
-		// Display posting list
-		cout << "Posting List for term `" << term << "' (termfreq "
-		     << db.get_termfreq(term) << ", collfreq "
-		     << db.get_collection_freq(term) << "):";
-		while (p != pend) {
-		    cout << separator << *p;
-		    if (verbose) {
-			cout << ' ' << p.get_wdf()
-			    << ' ' << p.get_doclength();
-		    }
-		    if (showvalues) show_values(db, *p, ' ');
-		    if (showdocdata) show_docdata(db, *p, ' ');
-		    p++;
-		}
-		cout << endl;
-	    } else {
-		// Display position lists
-		vector<docid>::const_iterator j;
-		for (j = recnos.begin(); j != recnos.end(); j++) {
-		    p.skip_to(*j);
-		    if (p == pend || *p != *j) {
-			cout << "term `" << term <<
-			    "' doesn't index document #" << *j << endl;
-		    } else {
-			cout << "Position List for term `" << term
-			    << "', record #" << *j << ':';
-			try {
-			    PositionIterator pos = p.positionlist_begin();
-			    PositionIterator posend = p.positionlist_end();
-			    while (pos != posend) {
-				cout << separator << *pos;
-				++pos;
-			    }
-			    cout << endl;
-			} catch (const Error &e) {
-			    cout << "Error: " << e.get_description() << endl;
-			}
-		    }
-		}
-	    }
-	}
-    } catch (const Error &e) {
-	cout << "\nError: " << e.get_description() << endl;
-	return 1;
     }
+
+    if (terms.empty()) {
+	show_termlists(db, recnos.begin(), recnos.end());
+	return 0;
+    }
+
+    vector<string>::const_iterator i;
+    for (i = terms.begin(); i != terms.end(); i++) {
+	string term = stemmer(*i);
+	PostingIterator p = db.postlist_begin(term);
+	PostingIterator pend = db.postlist_end(term);
+	if (p == pend) {
+	    cout << "term `" << term << "' not in database\n";
+	    continue;
+	}
+	if (recnos.empty()) {
+	    // Display posting list
+	    cout << "Posting List for term `" << term << "' (termfreq "
+		 << db.get_termfreq(term) << ", collfreq "
+		 << db.get_collection_freq(term) << ", wdf_max "
+		 << db.get_wdf_upper_bound(term) << "):";
+	    while (p != pend) {
+		cout << separator << *p;
+		if (verbose) {
+		    cout << ' ' << p.get_wdf() << ' ' << p.get_doclength();
+		}
+		if (showvalues) show_values(db, *p, ' ');
+		if (showdocdata) show_docdata(db, *p, ' ');
+		p++;
+	    }
+	    cout << endl;
+	} else {
+	    // Display position lists
+	    vector<docid>::const_iterator j;
+	    for (j = recnos.begin(); j != recnos.end(); j++) {
+		p.skip_to(*j);
+		if (p == pend || *p != *j) {
+		    cout << "term `" << term <<
+			"' doesn't index document #" << *j << endl;
+		} else {
+		    cout << "Position List for term `" << term
+			<< "', record #" << *j << ':';
+		    try {
+			PositionIterator pos = p.positionlist_begin();
+			PositionIterator posend = p.positionlist_end();
+			while (pos != posend) {
+			    cout << separator << *pos;
+			    ++pos;
+			}
+			cout << endl;
+		    } catch (const Error &e) {
+			cout << "Error: " << e.get_description() << endl;
+		    }
+		}
+	    }
+	}
+    }
+} catch (const Error &e) {
+    cout << "\nError: " << e.get_description() << endl;
+    return 1;
 }
