@@ -1,7 +1,7 @@
 /** @file xapian.h
  *  @brief Public interfaces for the Xapian library.
  */
-// Copyright (C) 2003,2004,2005,2007 Olly Betts
+// Copyright (C) 2003,2004,2005,2007,2008,2009 Olly Betts
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,21 +45,26 @@
 // Searching
 #include <xapian/enquire.h>
 #include <xapian/expanddecider.h>
+#include <xapian/keymaker.h>
+#include <xapian/matchspy.h>
+#include <xapian/postingsource.h>
 #include <xapian/query.h>
 #include <xapian/queryparser.h>
-#include <xapian/sorter.h>
+#include <xapian/valuesetmatchdecider.h>
+#include <xapian/weight.h>
 
 // Stemming
 #include <xapian/stem.h>
 
+// Subclass registry
+#include <xapian/registry.h>
+
 // Unicode support
 #include <xapian/unicode.h>
 
 // ELF visibility annotations for GCC.
 #include <xapian/visibility.h>
 
-#include <xapian/deprecated.h>
-
 /// The Xapian namespace contains public interfaces for the Xapian library.
 namespace Xapian {
 
@@ -73,14 +78,6 @@ namespace Xapian {
 XAPIAN_VISIBILITY_DEFAULT
 const char * version_string();
 
-/** For compatibility with Xapian 0.9.5 and earlier.
- *
- * @deprecated This function is now deprecated, use Xapian::version_string()
- * instead.
- */
-XAPIAN_VISIBILITY_DEFAULT
-XAPIAN_DEPRECATED(const char * xapian_version_string());
-
 /** Report the major version of the library which the program is linked with.
  *
  * This may be different to the version compiled against (given by
@@ -89,14 +86,6 @@ XAPIAN_DEPRECATED(const char * xapian_version_string());
 XAPIAN_VISIBILITY_DEFAULT
 int major_version();
 
-/** For compatibility with Xapian 0.9.5 and earlier.
- *
- * @deprecated This function is now deprecated, use Xapian::major_version()
- * instead.
- */
-XAPIAN_VISIBILITY_DEFAULT
-XAPIAN_DEPRECATED(int xapian_major_version());
-
 /** Report the minor version of the library which the program is linked with.
  *
  * This may be different to the version compiled against (given by
@@ -105,14 +94,6 @@ XAPIAN_DEPRECATED(int xapian_major_version());
 XAPIAN_VISIBILITY_DEFAULT
 int minor_version();
 
-/** For compatibility with Xapian 0.9.5 and earlier.
- *
- * @deprecated This function is now deprecated, use Xapian::minor_version()
- * instead.
- */
-XAPIAN_VISIBILITY_DEFAULT
-XAPIAN_DEPRECATED(int xapian_minor_version());
-
 /** Report the revision of the library which the program is linked with.
  *
  * This may be different to the version compiled against (given by
@@ -121,13 +102,6 @@ XAPIAN_DEPRECATED(int xapian_minor_version());
 XAPIAN_VISIBILITY_DEFAULT
 int revision();
 
-/** For compatibility with Xapian 0.9.5 and earlier.
- *
- * @deprecated This function is now deprecated, use Xapian::revision()
- * instead.
- */
-XAPIAN_VISIBILITY_DEFAULT
-XAPIAN_DEPRECATED(int xapian_revision());
 }
 
 #endif /* XAPIAN_INCLUDED_XAPIAN_H */
