@@ -29,16 +29,16 @@
 #include <vector>
 
 #include <xapian/base.h>
+#include <xapian/document.h>
 #include <xapian/types.h>
 #include <xapian/positioniterator.h>
 #include <xapian/postingiterator.h>
 #include <xapian/termiterator.h>
+#include <xapian/valueiterator.h>
 #include <xapian/visibility.h>
 
 namespace Xapian {
 
-class Document;
-
 /** This class is used to access a database, or a group of databases.
  *
  *  For searching, this class is used in conjunction with an Enquire object.
@@ -59,6 +59,23 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/// @private @internal Reference counted internals.
 	std::vector<Xapian::Internal::RefCntPtr<Internal> > internal;
 
+	/** @private @internal Get a document from the database, but doesn't
+	 *  need to check if it exists.
+	 *
+	 *  This method returns a Xapian::Document object which provides the
+	 *  information about a document.  If the document doesn't exist,
+	 *  either a NULL pointer may be returned, or the returned object will
+	 *  throw DocNotFoundError when you try to access it.
+	 *
+	 *  The caller should delete the returned object when it has finished
+	 *  with it.
+	 *
+	 *  @param did   The document id of the document to retrieve.
+	 *
+	 *  @return      Pointer to Document::Internal object.
+	 */
+	Document::Internal * get_document_lazily(Xapian::docid did) const;
+
 	/** Add an existing database (or group of databases) to those
 	 *  accessed by this object.
 	 *
@@ -99,12 +116,45 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	void operator=(const Database &other);
 
 	/** Re-open the database.
+	 *
 	 *  This re-opens the database(s) to the latest available version(s).
-	 *  It can be used either to make sure the latest results are
-	 *  returned, or to recover from a Xapian::DatabaseModifiedError.
+	 *  It can be used either to make sure the latest results are returned,
+	 *  or to recover from a Xapian::DatabaseModifiedError.
+	 *
+	 *  Calling reopen() on a database which has been closed (with @a
+	 *  close()) will always raise a Xapian::DatabaseError.
 	 */
 	void reopen();
 
+	/** Close the database.
+	 *
+	 *  This closes the database and releases all file handles held by the
+	 *  database.
+	 *
+	 *  This is a permanent close of the database: calling reopen() after
+	 *  closing a database will not reopen it, and will raise an exception.
+	 *
+	 *  Calling close() on a database which is already closed has no effect
+	 *  (and doesn't raise an exception).
+	 *
+	 *  After this call, calls made to methods of the database (other than
+	 *  close() or the destructor), or to objects associated with the
+	 *  database will behave in one of the following ways (but which
+	 *  behaviour happens may vary between releases, and between database
+	 *  backends):
+	 *
+	 *   - raise a Xapian::DatabaseError indicating that the database is
+	 *   closed.
+	 *
+	 *   - behave exactly as they would have done if the database had not
+	 *   been closed (by using cached data).
+	 *
+	 *  To summarise - you should not rely on the exception being raised,
+	 *  or the normal result being available, but if you do get a result,
+	 *  it will be correct.
+	 */
+	virtual void close();
+
 	/// Return a string describing this object.
 	virtual std::string get_description() const;
 
@@ -209,12 +259,68 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	Xapian::termcount get_collection_freq(const std::string & tname) const;
 
-	/** Get the length of a document.
+	/** Return the frequency of a given value slot.
+	 *
+	 *  This is the number of documents which have a (non-empty) value
+	 *  stored in the slot.
+	 *
+	 *  @param valno The value slot to examine.
+	 *
+	 *  @exception UnimplementedError The frequency of the value isn't
+	 *  available for this database type.
 	 */
-	Xapian::doclength get_doclength(Xapian::docid did) const;
+	Xapian::doccount get_value_freq(Xapian::valueno valno) const;
 
-	/** Send a "keep-alive" to remote databases to stop them timing
-	 *  out.
+	/** Get a lower bound on the values stored in the given value slot.
+	 *
+	 *  If there are no values stored in the given value slot, this will
+	 *  return an empty string.
+	 *
+	 *  If the lower bound isn't available for the given database type,
+	 *  this will return the lowest possible bound - the empty string.
+	 *
+	 *  @param valno The value slot to examine.
+	 */
+	std::string get_value_lower_bound(Xapian::valueno valno) const;
+
+	/** Get an upper bound on the values stored in the given value slot.
+	 *
+	 *  If there are no values stored in the given value slot, this will
+	 *  return an empty string.
+	 *
+	 *  @param valno The value slot to examine.
+	 *
+	 *  @exception UnimplementedError The upper bound of the values isn't
+	 *  available for this database type.
+	 */
+	std::string get_value_upper_bound(Xapian::valueno valno) const;
+
+	/** Get a lower bound on the length of a document in this DB.
+	 *
+	 *  This bound does not include any zero-length documents.
+	 */
+	Xapian::termcount get_doclength_lower_bound() const;
+
+	/// Get an upper bound on the length of a document in this DB.
+	Xapian::termcount get_doclength_upper_bound() const;
+
+	/// Get an upper bound on the wdf of term @a term.
+	Xapian::termcount get_wdf_upper_bound(const std::string & term) const;
+
+	/// Return an iterator over the value in slot @a slot for each document.
+	ValueIterator valuestream_begin(Xapian::valueno slot) const;
+
+	/// Return end iterator corresponding to valuestream_begin().
+	ValueIteratorEnd_ valuestream_end(Xapian::valueno) const {
+	    return ValueIteratorEnd_();
+	}
+
+	/// Get the length of a document.
+	Xapian::termcount get_doclength(Xapian::docid did) const;
+
+	/** Send a "keep-alive" to remote databases to stop them timing out.
+	 *
+	 *  Has no effect on non-remote databases.
 	 */
 	void keep_alive();
 
@@ -223,7 +329,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  This method returns a Xapian::Document object which provides the
 	 *  information about a document.
 	 *
-	 *  @param did   The document id for which to retrieve the data.
+	 *  @param did   The document id of the document to retrieve.
 	 *
 	 *  @return      A Xapian::Document object containing the document data
 	 *
@@ -273,10 +379,10 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  @param prefix   If non-empty, only terms with this prefix are
 	 *		    returned.
 	 */
-	Xapian::TermIterator synonym_keys_begin(const std::string &prefix = "") const;
+	Xapian::TermIterator synonym_keys_begin(const std::string &prefix = std::string()) const;
 
 	/// Corresponding end iterator to synonym_keys_begin(prefix).
-	Xapian::TermIterator synonym_keys_end(const std::string & = "") const {
+	Xapian::TermIterator synonym_keys_end(const std::string & = std::string()) const {
 	    return Xapian::TermIterator(NULL);
 	}
 
@@ -324,12 +430,28 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *			doesn't implement iterating its keys (currently
 	 *			this happens for the InMemory backend).
 	 */
-	Xapian::TermIterator metadata_keys_begin(const std::string &prefix = "") const;
+	Xapian::TermIterator metadata_keys_begin(const std::string &prefix = std::string()) const;
 
 	/// Corresponding end iterator to metadata_keys_begin().
-	Xapian::TermIterator metadata_keys_end(const std::string & = "") const {
+	Xapian::TermIterator metadata_keys_end(const std::string & = std::string()) const {
 	    return Xapian::TermIterator(NULL);
 	}
+
+	/** Get a UUID for the database.
+	 *
+	 *  The UUID will persist for the lifetime of the database.
+	 *
+	 *  Replicas (eg, made with the replication protocol, or by copying all
+	 *  the database files) will have the same UUID.  However, copies (made
+	 *  with copydatabase, or xapian-compact) will have different UUIDs.
+	 *
+	 *  If the backend does not support UUIDs or this database has no
+	 *  subdatabases, the UUID will be empty.
+	 *
+	 *  If this database has multiple sub-databases, the UUID string will
+	 *  contain the UUIDs of all the sub-databases.
+	 */
+	std::string get_uuid() const;
 };
 
 /** This class provides read/write access to a database.
@@ -390,17 +512,15 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 */
 	void operator=(const WritableDatabase &other);
 
-	/** Flush to disk any modifications made to the database.
+	/** Commit any pending modifications made to the database.
 	 *
 	 *  For efficiency reasons, when performing multiple updates to a
 	 *  database it is best (indeed, almost essential) to make as many
 	 *  modifications as memory will permit in a single pass through
 	 *  the database.  To ensure this, Xapian batches up modifications.
 	 *
-	 *  Flush may be called at any time to
-	 *  ensure that the modifications which have been made are written to
-	 *  disk: if the flush succeeds, all the preceding modifications will
-	 *  have been written to disk.
+	 *  This method may be called at any time to commit any pending
+	 *  modifications to the database.
 	 *
 	 *  If any of the modifications fail, an exception will be thrown and
 	 *  the database will be left in a state in which each separate
@@ -408,12 +528,12 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  performed or not performed at all: it is then up to the
 	 *  application to work out which operations need to be repeated.
 	 *
-	 *  It's not valid to call flush within a transaction.
+	 *  It's not valid to call commit() within a transaction.
 	 *
-	 *  Beware of calling flush too frequently: this will have a severe
-	 *  performance cost.
+	 *  Beware of calling commit() too frequently: this will make indexing
+	 *  take much longer.
 	 *
-	 *  Note that flush need not be called explicitly: it will be called
+	 *  Note that commit() need not be called explicitly: it will be called
 	 *  automatically when the database is closed, or when a sufficient
 	 *  number of modifications have been made.  By default, this is every
 	 *  10000 documents added, deleted, or modified.  This value is rather
@@ -427,7 +547,14 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  @exception Xapian::DatabaseCorruptError will be thrown if the
 	 *             database is in a corrupt state.
 	 */
-	void flush();
+	void commit();
+
+	/** Pre-1.1.0 name for commit().
+	 *
+	 *  Use commit() instead in new code.  This alias may be deprecated in
+	 *  the future.
+	 */
+	void flush() { commit(); }
 
 	/** Begin a transaction.
 	 *
@@ -441,18 +568,17 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  either be committed by calling commit_transaction() or aborted
 	 *  by calling cancel_transaction().
 	 *
-	 *  By default, a transaction implicitly calls flush before and after
-	 *  so that the modifications stand and fall without affecting
+	 *  By default, a transaction implicitly calls commit() before and
+	 *  after so that the modifications stand and fall without affecting
 	 *  modifications before or after.
 	 *
-	 *  The downside of this flushing is that small transactions cause
-	 *  modifications to be frequently flushed which can harm indexing
-	 *  performance in the same way that explicitly calling flush
-	 *  frequently can.
+	 *  The downside of these implicit calls to commit() is that small
+	 *  transactions can harm indexing performance in the same way that
+	 *  explicitly calling commit() frequently can.
 	 *
 	 *  If you're applying atomic groups of changes and only wish to
 	 *  ensure that each group is either applied or not applied, then
-	 *  you can prevent the automatic flush before and after the
+	 *  you can prevent the automatic commit() before and after the
 	 *  transaction by starting the transaction with
 	 *  begin_transaction(false).  However, if cancel_transaction is
 	 *  called (or if commit_transaction isn't called before the
@@ -498,7 +624,7 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	void commit_transaction();
 
 	/** Abort the transaction currently in progress, discarding the
-	 *  potential modifications made to the database.
+	 *  pending modifications made to the database.
 	 *
 	 *  If an error occurs in this method, an exception will be thrown,
 	 *  but the transaction will be cancelled anyway.
@@ -528,12 +654,12 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  call replace_document() instead.
 	 *
 	 *  Note that changes to the database won't be immediately committed to
-	 *  disk; see flush() for more details.
+	 *  disk; see commit() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully added, or the document
 	 *  fails to be added and an exception is thrown (possibly at a
-	 *  later time when flush is called or the database is closed).
+	 *  later time when commit() is called or the database is closed).
 	 *
 	 *  @param document The new document to be added.
 	 *
@@ -553,12 +679,12 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  from the database.
 	 *
 	 *  Note that changes to the database won't be immediately committed to
-	 *  disk; see flush() for more details.
+	 *  disk; see commit() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully removed, or the document
 	 *  fails to be removed and an exception is thrown (possibly at a
-	 *  later time when flush is called or the database is closed).
+	 *  later time when commit() is called or the database is closed).
 	 *
 	 *  @param did     The document ID of the document to be removed.
 	 *
@@ -606,12 +732,12 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  allocate document IDs!
 	 *
 	 *  Note that changes to the database won't be immediately committed to
-	 *  disk; see flush() for more details.
+	 *  disk; see commit() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully replaced, or the document
 	 *  fails to be replaced and an exception is thrown (possibly at a
-	 *  later time when flush is called or the database is closed).
+	 *  later time when commit() is called or the database is closed).
 	 *
 	 *  @param did     The document ID of the document to be replaced.
 	 *  @param document The new document.
@@ -632,20 +758,25 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  term, the lowest document ID will be used for the document,
 	 *  otherwise a new document ID will be generated as for add_document.
 	 *
-	 *  A major use is for convenience when UIDs from another system are
-	 *  mapped to terms in Xapian, although this method has other uses
-	 *  (for example, you could add a "deletion date" term to documents at
-	 *  index time and use this method to delete all documents due for
-	 *  deletion on a particular date).
+	 *  One common use is to allow UIDs from another system to easily be
+	 *  mapped to terms in Xapian.  Note that this method doesn't
+	 *  automatically add unique_term as a term, so you'll need to call
+	 *  document.add_term(unique_term) first when using replace_document()
+	 *  in this way.
+	 *
+	 *  Another possible use is to allow groups of documents to be marked for
+	 *  later deletion - for example, you could add a "deletion date" term
+	 *  to documents at index time and use this method to easily and efficiently
+	 *  delete all documents due for deletion on a particular date.
 	 *
 	 *  Note that changes to the database won't be immediately committed to
-	 *  disk; see flush() for more details.
+	 *  disk; see commit() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document(s) will either be fully replaced, or the
 	 *  document(s) fail to be replaced and an exception is thrown
 	 *  (possibly at a
-	 *  later time when flush is called or the database is closed).
+	 *  later time when commit() is called or the database is closed).
 	 *
 	 *  @param unique_term    The "unique" term.
 	 *  @param document The new document.
@@ -722,7 +853,7 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  Metadata modifications are committed to disk in the same way as
 	 *  modifications to the documents in the database are: i.e.,
 	 *  modifications are atomic, and won't be committed to disk
-	 *  immediately (see flush() for more details).  This allows metadata
+	 *  immediately (see commit() for more details).  This allows metadata
 	 *  to be used to link databases with versioned external resources
 	 *  by storing the appropriate version number in a metadata item.
 	 *
