@@ -4,6 +4,7 @@
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2006,2007,2009,2010 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,12 +30,11 @@
 #include <xapian/base.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
+#include <xapian/valueiterator.h>
 #include <xapian/visibility.h>
 
 namespace Xapian {
 
-class ValueIterator;
-
 /** A handle representing a document in a Xapian database.
  *
  *  The Document class fetches information from the database lazily.  Usually
@@ -47,7 +47,11 @@ class ValueIterator;
  *  situation above can only happen with a WritableDatabase object, or if
  *  you call Database::reopen() on a Database object.
  *
- *  We recommend you avoid designs where this behaviour is an issue.
+ *  We recommend you avoid designs where this behaviour is an issue, but if
+ *  you need a way to make a non-lazy version of a Document object, you can do
+ *  this like so:
+ *
+ *      doc = Xapian::Document::unserialise(doc.serialise());
  */
 class XAPIAN_VISIBILITY_DEFAULT Document {
     public:
@@ -212,7 +216,9 @@ class XAPIAN_VISIBILITY_DEFAULT Document {
 	ValueIterator values_begin() const;
 
 	/// Equivalent end iterator for values_begin().
-	ValueIterator values_end() const;
+	ValueIteratorEnd_ values_end() const {
+	    return ValueIteratorEnd_();
+	}
 
 	/** Get the document id which is associated with this document (if any).
 	 *
@@ -225,6 +231,19 @@ class XAPIAN_VISIBILITY_DEFAULT Document {
 	 */
 	docid get_docid() const;
 
+	/** Serialise document into a string.
+	 *
+	 *  The document representation may change between Xapian releases:
+	 *  even between minor versions.  However, it is guaranteed not to
+	 *  change if the remote database protocol has not changed between
+	 *  releases.
+	 */
+	std::string serialise() const;
+
+	/** Unserialise a document from a string produced by serialise().
+	 */
+	static Document unserialise(const std::string &s);
+
 	/// Return a string describing this object.
 	std::string get_description() const;
 };
