@@ -3,8 +3,8 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006,2007,2009 Olly Betts
- * Copyright 2006,2007,2008 Lemur Consulting Ltd
+ * Copyright 2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2006,2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,7 +29,6 @@
 #include <vector>
 
 #include <xapian/base.h>
-#include <xapian/deprecated.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
 #include <xapian/visibility.h>
@@ -44,6 +43,9 @@ struct SortPosName;
 
 namespace Xapian {
 
+class PostingSource;
+class Registry;
+
 /** Class representing a query.
  *
  *  Queries are represented as a tree of objects.
@@ -119,7 +121,24 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	    OP_VALUE_GE,
 
 	    /** Filter by a less-than-or-equal test on a document value. */
-	    OP_VALUE_LE
+	    OP_VALUE_LE,
+
+	    /** Treat a set of queries as synonyms.
+	     *
+	     *  This returns all results which match at least one of the
+	     *  queries, but weighting as if all the sub-queries are instances
+	     *  of the same term: so multiple matching terms for a document
+	     *  increase the wdf value used, and the term frequency is based on
+	     *  the number of documents which would match an OR of all the
+	     *  subqueries.
+	     *
+	     *  The term frequency used will usually be an approximation,
+	     *  because calculating the precise combined term frequency would
+	     *  be overly expensive.
+	     *
+	     *  Identical to OP_OR, except for the weightings returned.
+	     */
+	    OP_SYNONYM
 	} op;
 
 	/** Copy constructor. */
@@ -157,7 +176,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  The Xapian::Query objects are specified with begin and end
 	 *  iterators.
 	 * 
-	 *  AND, OR, NEAR and PHRASE can take any number of subqueries.
+	 *  AND, OR, SYNONYM, NEAR and PHRASE can take any number of subqueries.
 	 *  Other operators take exactly two subqueries.
 	 *
 	 *  The iterators may be to Xapian::Query objects, pointers to
@@ -171,14 +190,6 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	Query(Query::op op_, Iterator qbegin, Iterator qend,
 	      Xapian::termcount parameter = 0);
 
-	/** Apply the specified operator to a single Xapian::Query object.
-	 *
-	 *  @deprecated This method is deprecated because it isn't useful,
-	 *  since none of the current query operators can be usefully
-	 *  applied to a single subquery without a parameter value.
-	 */
-	XAPIAN_DEPRECATED(Query(Query::op op_, Xapian::Query q));
-
 	/** Apply the specified operator to a single Xapian::Query object, with
 	 *  a double parameter.
 	 */
@@ -213,11 +224,23 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 */
 	Query(Query::op op_, Xapian::valueno valno, const std::string &value);
 
+	/** Construct an external source query.
+	 *
+	 *  An attempt to clone the posting source will be made immediately, so
+	 *  if the posting source supports clone(), the source supplied may be
+	 *  safely deallocated after this call.  If the source does not support
+	 *  clone(), the caller must ensure that the posting source remains
+	 *  valid until the Query is deallocated.
+	 *
+	 *  @param external_source The source to use in the query.
+	 */
+	explicit Query(Xapian::PostingSource * external_source);
+
 	/** A query which matches all documents in the database. */
-	static Xapian::Query MatchAll;
+	static const Xapian::Query MatchAll;
 
 	/** A query which matches no documents. */
-	static Xapian::Query MatchNothing;
+	static const Xapian::Query MatchNothing;
 
 	/** Get the length of the query, used by some ranking formulae.
 	 *  This value is calculated automatically - if you want to override
@@ -244,6 +267,37 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 */
 	bool empty() const;
 
+	/** Serialise query into a string.
+	 *
+	 *  The query representation may change between Xapian releases:
+	 *  even between minor versions.  However, it is guaranteed not to
+	 *  change unless the remote database protocol has also changed between
+	 *  releases.
+	 */
+	std::string serialise() const;
+
+	/** Unserialise a query from a string produced by serialise().
+	 *
+	 *  This method will fail if the query contains any external
+	 *  PostingSource leaf nodes.
+	 *
+	 *  @param s The string representing the serialised query.
+	 */
+	static Query unserialise(const std::string &s);
+
+	/** Unserialise a query from a string produced by serialise().
+	 *
+	 *  The supplied registry will be used to attempt to unserialise any
+	 *  external PostingSource leaf nodes.  This method will fail if the
+	 *  query contains any external PostingSource leaf nodes which are not
+	 *  registered in the registry.
+	 *
+	 *  @param s The string representing the serialised query.
+	 *  @param registry Xapian::Registry to use.
+	 */
+	static Query unserialise(const std::string & s,
+				 const Registry & registry);
+
 	/// Return a string describing this object.
 	std::string get_description() const;
 
@@ -276,7 +330,9 @@ Query::Query(Query::op op_, Iterator qbegin, Iterator qend, termcount parameter)
     }
 }
 
-/// @internal Internal class, implementing most of Xapian::Query.
+#ifndef SWIG // SWIG has no interest in the internal class, so hide it completely.
+
+/// @private @internal Internal class, implementing most of Xapian::Query.
 class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCntBase {
     friend class ::LocalSubMatch;
     friend class ::MultiMatch;
@@ -285,6 +341,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
     friend class Query;
     public:
         static const int OP_LEAF = -1;
+        static const int OP_EXTERNAL_SOURCE = -2;
 
 	/// The container type for storing pointers to subqueries
 	typedef std::vector<Internal *> subquery_list;
@@ -305,6 +362,8 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	 * For ELITE_SET, the number of terms to select from those specified.
 	 *
 	 * For RANGE, the value number to apply the range test to.
+	 *
+	 * For a leaf node, this is the within query frequency of the term.
 	 */
 	Xapian::termcount parameter;
 
@@ -322,8 +381,11 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	/// Position in query of this term - leaf node only
 	Xapian::termpos term_pos;
 
-	/// Within query frequency of this term - leaf node only
-	Xapian::termcount wqf;
+	/// External posting source.
+	Xapian::PostingSource * external_source;
+
+	/// Flag, indicating whether the external source is owned by the query.
+	bool external_source_owned;
 
 	/// Copy another Xapian::Query::Internal into self.
 	void initialise_from_copy(const Query::Internal & copyme);
@@ -337,10 +399,10 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	 */
 	Internal * simplify_query();
 
-	/** Perform checks that query is valid. (eg, has correct number of
+	/** Perform checks that query is valid. (e.g., has correct number of
 	 *  sub queries.)  Throw an exception if not.  This is initially called
 	 *  on the query before any simplifications have been made, and after
-	 *  simplications.
+	 *  simplifications.
 	 */
 	void validate_query() const;
 
@@ -390,10 +452,14 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	 */
 	Internal(op_t op_, Xapian::valueno valno, const std::string &value);
 
+	/// Construct an external source query.
+	explicit Internal(Xapian::PostingSource * external_source_, bool owned);
+
 	/** Destructor. */
 	~Internal();
 
-	static Xapian::Query::Internal * unserialise(const std::string &s);
+	static Xapian::Query::Internal * unserialise(const std::string &s,
+						     const Registry & registry);
 
 	/** Add a subquery. */
 	void add_subquery(const Query::Internal * subq);
@@ -432,6 +498,8 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	 */
 	Xapian::termcount get_parameter() const { return parameter; }
 
+	Xapian::termcount get_wqf() const { return parameter; }
+
 	/** Get the length of the query, used by some ranking formulae.
 	 *  This value is calculated automatically - if you want to override
 	 *  it you can pass a different value to Enquire::set_query().
@@ -446,6 +514,8 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	TermIterator get_terms() const;
 };
 
+#endif // SWIG
+
 }
 
 #endif /* XAPIAN_INCLUDED_QUERY_H */
