@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005 Olly Betts
+ * Copyright 2003,2004,2005,2008,2009 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,16 +22,16 @@
  */
 
 #include <config.h>
-
 #include "andmaybepostlist.h"
-#include "andpostlist.h"
+
+#include "debuglog.h"
+#include "multiandpostlist.h"
 #include "omassert.h"
-#include "omdebug.h"
 
 PostList *
 AndMaybePostList::process_next_or_skip_to(Xapian::weight w_min, PostList *ret)
 {
-    DEBUGCALL(MATCH, PostList *, "AndMaybePostList::process_next_or_skip_to", w_min << ", " << ret);
+    LOGCALL(MATCH, PostList *, "AndMaybePostList::process_next_or_skip_to", w_min | ret);
     handle_prune(l, ret);
     if (l->at_end()) {
 	// once l is over, so is the AND MAYBE
@@ -41,25 +42,49 @@ AndMaybePostList::process_next_or_skip_to(Xapian::weight w_min, PostList *ret)
     lhead = l->get_docid();
     if (lhead <= rhead) RETURN(NULL);
 
-    skip_to_handling_prune(r, lhead, w_min - lmax, matcher);
+    bool valid;
+    check_handling_prune(r, lhead, w_min - lmax, matcher, valid);
     if (r->at_end()) {
 	PostList *tmp = l;
 	l = NULL;
 	RETURN(tmp);
     }
-    rhead = r->get_docid();
+    if (valid) {
+	rhead = r->get_docid();
+    } else {
+	rhead = 0;
+    }
+    RETURN(NULL);
+}
+
+PostList *
+AndMaybePostList::sync_rhs(Xapian::weight w_min)
+{
+    LOGCALL(MATCH, PostList *, "AndMaybePostList::sync_rhs", w_min);
+    bool valid;
+    check_handling_prune(r, lhead, w_min - lmax, matcher, valid);
+    if (r->at_end()) {
+	PostList *tmp = l;
+	l = NULL;
+	RETURN(tmp);
+    }
+    if (valid) {
+	rhead = r->get_docid();
+    } else {
+	rhead = 0;
+    }
     RETURN(NULL);
 }
 
 PostList *
 AndMaybePostList::next(Xapian::weight w_min)
 {
-    DEBUGCALL(MATCH, PostList *, "AndMaybePostList::next", w_min);
+    LOGCALL(MATCH, PostList *, "AndMaybePostList::next", w_min);
     if (w_min > lmax) {
 	// we can replace the AND MAYBE with an AND
 	PostList *ret;
-	DEBUGLINE(MATCH, "AND MAYBE -> AND");
-	ret = new AndPostList(l, r, matcher, dbsize, true);
+	LOGLINE(MATCH, "AND MAYBE -> AND");
+	ret = new MultiAndPostList(l, r, lmax, rmax, matcher, dbsize, true);
 	l = r = NULL;
 	skip_to_handling_prune(ret, std::max(lhead, rhead) + 1, w_min, matcher);
 	RETURN(ret);
@@ -70,12 +95,12 @@ AndMaybePostList::next(Xapian::weight w_min)
 PostList *
 AndMaybePostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 {
-    DEBUGCALL(MATCH, PostList *, "AndMaybePostList::skip_to", did << ", " << w_min);
+    LOGCALL(MATCH, PostList *, "AndMaybePostList::skip_to", did | w_min);
     if (w_min > lmax) {
 	// we can replace the AND MAYBE with an AND
 	PostList *ret;
-	DEBUGLINE(MATCH, "AND MAYBE -> AND (in skip_to)");
-	ret = new AndPostList(l, r, matcher, dbsize, true);
+	LOGLINE(MATCH, "AND MAYBE -> AND (in skip_to)");
+	ret = new MultiAndPostList(l, r, lmax, rmax, matcher, dbsize, true);
 	did = std::max(did, std::max(lhead, rhead));
 	l = r = NULL;
 	skip_to_handling_prune(ret, did, w_min, matcher);
@@ -91,7 +116,7 @@ AndMaybePostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 Xapian::doccount
 AndMaybePostList::get_termfreq_max() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_max", "");
+    LOGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_max", NO_ARGS);
     // Termfreq is exactly that of left hand branch.
     RETURN(l->get_termfreq_max());
 }
@@ -99,7 +124,7 @@ AndMaybePostList::get_termfreq_max() const
 Xapian::doccount
 AndMaybePostList::get_termfreq_min() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_min", "");
+    LOGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_min", NO_ARGS);
     // Termfreq is exactly that of left hand branch.
     RETURN(l->get_termfreq_min());
 }
@@ -107,16 +132,25 @@ AndMaybePostList::get_termfreq_min() const
 Xapian::doccount
 AndMaybePostList::get_termfreq_est() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_est", "");
+    LOGCALL(MATCH, Xapian::doccount, "AndMaybePostList::get_termfreq_est", NO_ARGS);
     // Termfreq is exactly that of left hand branch.
     RETURN(l->get_termfreq_est());
 }
 
+TermFreqs
+AndMaybePostList::get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const
+{
+    LOGCALL(MATCH, TermFreqs, "AndMaybePostList::get_termfreq_est_using_stats", stats);
+    // Termfreq is exactly that of left hand branch.
+    RETURN(l->get_termfreq_est_using_stats(stats));
+}
+
 Xapian::docid
 AndMaybePostList::get_docid() const
 {
-    DEBUGCALL(MATCH, Xapian::docid, "AndMaybePostList::get_docid", "");
-    Assert(lhead != 0 && rhead != 0); // check we've started
+    LOGCALL(MATCH, Xapian::docid, "AndMaybePostList::get_docid", NO_ARGS);
+    Assert(lhead != 0); // check we've started
     RETURN(lhead);
 }
 
@@ -124,8 +158,8 @@ AndMaybePostList::get_docid() const
 Xapian::weight
 AndMaybePostList::get_weight() const
 {
-    DEBUGCALL(MATCH, Xapian::weight, "AndMaybePostList::get_weight", "");
-    Assert(lhead != 0 && rhead != 0); // check we've started
+    LOGCALL(MATCH, Xapian::weight, "AndMaybePostList::get_weight", NO_ARGS);
+    Assert(lhead != 0); // check we've started
     if (lhead == rhead) RETURN(l->get_weight() + r->get_weight());
     RETURN(l->get_weight());
 }
@@ -134,14 +168,14 @@ AndMaybePostList::get_weight() const
 Xapian::weight
 AndMaybePostList::get_maxweight() const
 {
-    DEBUGCALL(MATCH, Xapian::weight, "AndMaybePostList::get_maxweight", "");
+    LOGCALL(MATCH, Xapian::weight, "AndMaybePostList::get_maxweight", NO_ARGS);
     RETURN(lmax + rmax);
 }
 
 Xapian::weight
 AndMaybePostList::recalc_maxweight()
 {
-    DEBUGCALL(MATCH, Xapian::weight, "AndMaybePostList::recalc_maxweight", "");
+    LOGCALL(MATCH, Xapian::weight, "AndMaybePostList::recalc_maxweight", NO_ARGS);
     lmax = l->recalc_maxweight();
     rmax = r->recalc_maxweight();
     RETURN(AndMaybePostList::get_maxweight());
@@ -150,7 +184,7 @@ AndMaybePostList::recalc_maxweight()
 bool
 AndMaybePostList::at_end() const
 {
-    DEBUGCALL(MATCH, bool, "AndMaybePostList::at_end", "");
+    LOGCALL(MATCH, bool, "AndMaybePostList::at_end", NO_ARGS);
     RETURN(lhead == 0);
 }
 
@@ -161,11 +195,28 @@ AndMaybePostList::get_description() const
 	   ")";
 }
 
-Xapian::doclength
+Xapian::termcount
 AndMaybePostList::get_doclength() const
 {
-    DEBUGCALL(MATCH, Xapian::doclength, "AndMaybePostList::get_doclength", "");
-    Assert(lhead != 0 && rhead != 0); // check we've started
-    if (lhead == rhead) AssertEqDouble(l->get_doclength(), r->get_doclength());
+    LOGCALL(MATCH, Xapian::termcount, "AndMaybePostList::get_doclength", NO_ARGS);
+    Assert(lhead != 0); // check we've started
+    if (lhead == rhead) AssertEq(l->get_doclength(), r->get_doclength());
     RETURN(l->get_doclength());
 }
+
+Xapian::termcount
+AndMaybePostList::get_wdf() const
+{
+    LOGCALL(MATCH, Xapian::termcount, "AndMaybePostList::get_wdf", NO_ARGS);
+    if (lhead == rhead) RETURN(l->get_wdf() + r->get_wdf());
+    RETURN(l->get_wdf());
+}
+
+Xapian::termcount
+AndMaybePostList::count_matching_subqs() const
+{
+    LOGCALL(MATCH, Xapian::termcount, "AndMaybePostList::count_matching_subqs", NO_ARGS);
+    if (lhead == rhead)
+	RETURN(l->count_matching_subqs() + r->count_matching_subqs());
+    RETURN(l->count_matching_subqs());
+}
