@@ -2,7 +2,9 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2003,2004,2007 Olly Betts
+ * Copyright 2003,2004,2007,2008,2009 Olly Betts
+ * Copyright 2009 Lemur Consulting Ltd
+ * Copyright 2010 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +23,12 @@
  */
 
 #include <config.h>
-
 #include "orpostlist.h"
+
+#include "debuglog.h"
 #include "multiandpostlist.h"
 #include "andmaybepostlist.h"
 #include "omassert.h"
-#include "omdebug.h"
 
 #include <algorithm>
 
@@ -37,34 +39,51 @@ OrPostList::OrPostList(PostList *left_,
 	: BranchPostList(left_, right_, matcher_),
 	  lhead(0), rhead(0), lmax(0), rmax(0), minmax(0), dbsize(dbsize_)
 {
-    DEBUGCALL(MATCH, void, "OrPostList", left_ << ", " << right_ << ", " << matcher_ << ", " << dbsize_);
+    LOGCALL_CTOR(MATCH, "OrPostList", left_ | right_ | matcher_ | dbsize_);
     AssertRel(left_->get_termfreq_est(),>=,right_->get_termfreq_est());
 }
 
 PostList *
 OrPostList::next(Xapian::weight w_min)
 {
-    DEBUGCALL(MATCH, PostList *, "OrPostList::next", w_min);
+    LOGCALL(MATCH, PostList *, "OrPostList::next", w_min);
     if (w_min > minmax) {
 	// we can replace the OR with another operator
 	PostList *ret;
 	if (w_min > lmax) {
 	    if (w_min > rmax) {
-		DEBUGLINE(MATCH, "OR -> AND");
+		LOGLINE(MATCH, "OR -> AND");
 		ret = new MultiAndPostList(l, r, lmax, rmax, matcher, dbsize);
-		skip_to_handling_prune(ret, std::max(lhead, rhead) + 1, w_min,
-				       matcher);
+		Xapian::docid newdocid = std::max(lhead, rhead);
+		if (lhead == rhead) {
+		    ++newdocid;
+		}
+		skip_to_handling_prune(ret, newdocid, w_min, matcher);
 	    } else {
-		DEBUGLINE(MATCH, "OR -> AND MAYBE (1)");
-		ret = new AndMaybePostList(r, l, matcher, dbsize, rhead, lhead);
-		next_handling_prune(ret, w_min, matcher);
+		LOGLINE(MATCH, "OR -> AND MAYBE (1)");
+		AndMaybePostList * ret2 =
+		    new AndMaybePostList(r, l, matcher, dbsize, rhead, lhead);
+		ret = ret2;
+		// Advance the AndMaybePostList unless the old RHS postlist was
+		// already ahead of the current docid.
+		if (rhead <= lhead) {
+		    next_handling_prune(ret, w_min, matcher);
+		} else {
+		    handle_prune(ret, ret2->sync_rhs(w_min));
+		}
 	    }
 	} else {
 	    // w_min > rmax since w_min > minmax but not (w_min > lmax)
 	    Assert(w_min > rmax);
-	    DEBUGLINE(MATCH, "OR -> AND MAYBE (2)");
-	    ret = new AndMaybePostList(l, r, matcher, dbsize, lhead, rhead);
-	    next_handling_prune(ret, w_min, matcher);
+	    LOGLINE(MATCH, "OR -> AND MAYBE (2)");
+	    AndMaybePostList * ret2 =
+		    new AndMaybePostList(l, r, matcher, dbsize, lhead, rhead);
+	    ret = ret2;
+	    if (lhead <= rhead) {
+		next_handling_prune(ret, w_min, matcher);
+	    } else {
+		handle_prune(ret, ret2->sync_rhs(w_min));
+	    }
 	}
 
 	l = r = NULL;
@@ -105,25 +124,31 @@ OrPostList::next(Xapian::weight w_min)
 PostList *
 OrPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 {
-    DEBUGCALL(MATCH, PostList *, "OrPostList::skip_to", did << ", " << w_min);
+    LOGCALL(MATCH, PostList *, "OrPostList::skip_to", did | w_min);
     if (w_min > minmax) {
 	// we can replace the OR with another operator
 	PostList *ret;
 	if (w_min > lmax) {
 	    if (w_min > rmax) {
-		DEBUGLINE(MATCH, "OR -> AND (in skip_to)");
+		LOGLINE(MATCH, "OR -> AND (in skip_to)");
 		ret = new MultiAndPostList(l, r, lmax, rmax, matcher, dbsize);
 		did = std::max(did, std::max(lhead, rhead));
 	    } else {
-		DEBUGLINE(MATCH, "OR -> AND MAYBE (in skip_to) (1)");
-		ret = new AndMaybePostList(r, l, matcher, dbsize, rhead, lhead);
+		LOGLINE(MATCH, "OR -> AND MAYBE (in skip_to) (1)");
+		AndMaybePostList * ret2 =
+			new AndMaybePostList(r, l, matcher, dbsize, rhead, lhead);
+		ret = ret2;
+		handle_prune(ret, ret2->sync_rhs(w_min));
 		did = std::max(did, rhead);
 	    }
 	} else {
 	    // w_min > rmax since w_min > minmax but not (w_min > lmax)
 	    Assert(w_min > rmax);
-	    DEBUGLINE(MATCH, "OR -> AND MAYBE (in skip_to) (2)");
-	    ret = new AndMaybePostList(l, r, matcher, dbsize, lhead, rhead);
+	    LOGLINE(MATCH, "OR -> AND MAYBE (in skip_to) (2)");
+	    AndMaybePostList * ret2 =
+		    new AndMaybePostList(l, r, matcher, dbsize, lhead, rhead);
+	    ret = ret2;
+	    handle_prune(ret, ret2->sync_rhs(w_min));
 	    did = std::max(did, lhead);
 	}
 
@@ -159,24 +184,105 @@ OrPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
     RETURN(ret);
 }
 
+PostList *
+OrPostList::check(Xapian::docid did, Xapian::weight w_min, bool &valid)
+{
+    LOGCALL(MATCH, PostList *, "OrPostList::check", did | w_min);
+    if (w_min > minmax) {
+	// we can replace the OR with another operator
+	PostList *ret;
+	if (w_min > lmax) {
+	    if (w_min > rmax) {
+		LOGLINE(MATCH, "OR -> AND (in check)");
+		ret = new MultiAndPostList(l, r, lmax, rmax, matcher, dbsize);
+		did = std::max(did, std::max(lhead, rhead));
+	    } else {
+		LOGLINE(MATCH, "OR -> AND MAYBE (in check) (1)");
+		AndMaybePostList * ret2 = new AndMaybePostList(r, l, matcher, dbsize, rhead, lhead);
+		ret = ret2;
+		handle_prune(ret, ret2->sync_rhs(w_min));
+		did = std::max(did, rhead);
+	    }
+	} else {
+	    // w_min > rmax since w_min > minmax but not (w_min > lmax)
+	    Assert(w_min > rmax);
+	    LOGLINE(MATCH, "OR -> AND MAYBE (in check) (2)");
+	    AndMaybePostList * ret2 = new AndMaybePostList(l, r, matcher, dbsize, lhead, rhead);
+	    ret = ret2;
+	    handle_prune(ret, ret2->sync_rhs(w_min));
+	    did = std::max(did, lhead);
+	}
+
+	l = r = NULL;
+	check_handling_prune(ret, did, w_min, matcher, valid);
+	RETURN(ret);
+    }
+
+    bool ldry = false;
+    bool lvalid = false;
+    bool rvalid = false;
+    if (lhead <= did) {
+	check_handling_prune(l, did, w_min - rmax, matcher, lvalid);
+	ldry = l->at_end();
+    }
+
+    if (rhead <= did) {
+	check_handling_prune(r, did, w_min - lmax, matcher, rvalid);
+
+	if (r->at_end()) {
+	    PostList *ret = l;
+	    l = NULL;
+	    valid = lvalid;
+	    RETURN(ret);
+	}
+	if (rvalid) {
+	    rhead = r->get_docid();
+	} else {
+	    rhead = did + 1;
+	}
+    }
+
+    if (!ldry) {
+	if (lvalid) {
+	    lhead = l->get_docid();
+	} else {
+	    lhead = did + 1;
+	}
+
+	if (lhead < rhead) {
+	    valid = lvalid;
+	} else if (rhead < lhead) {
+	    valid = rvalid;
+	} else {
+	    valid = lvalid || rvalid;
+	}
+	RETURN(NULL);
+    }
+
+    PostList *ret = r;
+    r = NULL;
+    valid = rvalid;
+    RETURN(ret);
+}
+
 Xapian::doccount
 OrPostList::get_termfreq_max() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_max", "");
+    LOGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_max", NO_ARGS);
     RETURN(std::min(l->get_termfreq_max() + r->get_termfreq_max(), dbsize));
 }
 
 Xapian::doccount
 OrPostList::get_termfreq_min() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_min", "");
+    LOGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_min", NO_ARGS);
     RETURN(std::max(l->get_termfreq_min(), r->get_termfreq_min()));
 }
 
 Xapian::doccount
 OrPostList::get_termfreq_est() const
 {
-    DEBUGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_est", "");
+    LOGCALL(MATCH, Xapian::doccount, "OrPostList::get_termfreq_est", NO_ARGS);
     // Estimate assuming independence:
     // P(l or r) = P(l) + P(r) - P(l) . P(r)
     double lest = static_cast<double>(l->get_termfreq_est());
@@ -185,10 +291,39 @@ OrPostList::get_termfreq_est() const
     RETURN(static_cast<Xapian::doccount>(est + 0.5));
 }
 
+TermFreqs
+OrPostList::get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const 
+{
+    LOGCALL(MATCH, TermFreqs, "OrPostList::get_termfreq_est_using_stats", stats);
+    // Estimate assuming independence:
+    // P(l or r) = P(l) + P(r) - P(l) . P(r)
+    TermFreqs lfreqs(l->get_termfreq_est_using_stats(stats));
+    TermFreqs rfreqs(r->get_termfreq_est_using_stats(stats));
+
+    double freqest, relfreqest;
+
+    // Our caller should have ensured this.
+    Assert(stats.collection_size);
+
+    freqest = lfreqs.termfreq + rfreqs.termfreq -
+	    (lfreqs.termfreq * rfreqs.termfreq / stats.collection_size);
+
+    if (stats.rset_size == 0) {
+	relfreqest = 0;
+    } else {
+	relfreqest = lfreqs.reltermfreq + rfreqs.reltermfreq -
+		(lfreqs.reltermfreq * rfreqs.reltermfreq / stats.rset_size);
+    }
+
+    RETURN(TermFreqs(static_cast<Xapian::doccount>(freqest + 0.5),
+		     static_cast<Xapian::doccount>(relfreqest + 0.5)));
+}
+
 Xapian::docid
 OrPostList::get_docid() const
 {
-    DEBUGCALL(MATCH, Xapian::docid, "OrPostList::get_docid", "");
+    LOGCALL(MATCH, Xapian::docid, "OrPostList::get_docid", NO_ARGS);
     Assert(lhead != 0 && rhead != 0); // check we've started
     RETURN(std::min(lhead, rhead));
 }
@@ -197,7 +332,7 @@ OrPostList::get_docid() const
 Xapian::weight
 OrPostList::get_weight() const
 {
-    DEBUGCALL(MATCH, Xapian::weight, "OrPostList::get_weight", "");
+    LOGCALL(MATCH, Xapian::weight, "OrPostList::get_weight", NO_ARGS);
     Assert(lhead != 0 && rhead != 0); // check we've started
     if (lhead < rhead) RETURN(l->get_weight());
     if (lhead > rhead) RETURN(r->get_weight());
@@ -208,14 +343,17 @@ OrPostList::get_weight() const
 Xapian::weight
 OrPostList::get_maxweight() const
 {
-    DEBUGCALL(MATCH, Xapian::weight, "OrPostList::get_maxweight", "");
+    LOGCALL(MATCH, Xapian::weight, "OrPostList::get_maxweight", NO_ARGS);
     RETURN(lmax + rmax);
 }
 
 Xapian::weight
 OrPostList::recalc_maxweight()
 {
-    DEBUGCALL(MATCH, Xapian::weight, "OrPostList::recalc_maxweight", "");
+    LOGCALL(MATCH, Xapian::weight, "OrPostList::recalc_maxweight", NO_ARGS);
+    // l and r cannot be NULL here, because the only place where they get set
+    // to NULL is when the tree is decaying, and the OrPostList is then
+    // immediately replaced.
     lmax = l->recalc_maxweight();
     rmax = r->recalc_maxweight();
     minmax = std::min(lmax, rmax);
@@ -225,7 +363,7 @@ OrPostList::recalc_maxweight()
 bool
 OrPostList::at_end() const
 {
-    DEBUGCALL(MATCH, bool, "OrPostList::at_end", "");
+    LOGCALL(MATCH, bool, "OrPostList::at_end", NO_ARGS);
     // Can never really happen - OrPostList next/skip_to autoprune
     AssertParanoid(!(l->at_end()) && !(r->at_end()));
     RETURN(false);
@@ -237,22 +375,40 @@ OrPostList::get_description() const
     return "(" + l->get_description() + " Or " + r->get_description() + ")";
 }
 
-Xapian::doclength
+Xapian::termcount
 OrPostList::get_doclength() const
 {
-    DEBUGCALL(MATCH, Xapian::doclength, "OrPostList::get_doclength", "");
-    Xapian::doclength doclength;
+    LOGCALL(MATCH, Xapian::termcount, "OrPostList::get_doclength", NO_ARGS);
+    Xapian::termcount doclength;
 
     Assert(lhead != 0 && rhead != 0); // check we've started
     if (lhead > rhead) {
 	doclength = r->get_doclength();
-	DEBUGLINE(MATCH, "OrPostList::get_doclength() [right docid=" 
-		  << rhead << "] = " << doclength);
+	LOGLINE(MATCH, "OrPostList::get_doclength() [right docid=" << rhead <<
+		       "] = " << doclength);
     } else {
 	doclength = l->get_doclength();
-	DEBUGLINE(MATCH, "OrPostList::get_doclength() [left docid="
-		  << lhead << "] = " << doclength);
+	LOGLINE(MATCH, "OrPostList::get_doclength() [left docid=" << lhead <<
+	       	       "] = " << doclength);
     }
 
     RETURN(doclength);
 }
+
+Xapian::termcount
+OrPostList::get_wdf() const
+{
+    LOGCALL(MATCH, Xapian::termcount, "OrPostList::get_wdf", NO_ARGS);
+    if (lhead < rhead) RETURN(l->get_wdf());
+    if (lhead > rhead) RETURN(r->get_wdf());
+    RETURN(l->get_wdf() + r->get_wdf());
+}
+
+Xapian::termcount
+OrPostList::count_matching_subqs() const
+{
+    LOGCALL(MATCH, Xapian::termcount, "OrPostList::count_matching_subqs", NO_ARGS);
+    if (lhead < rhead) RETURN(l->count_matching_subqs());
+    if (lhead > rhead) RETURN(r->count_matching_subqs());
+    RETURN(l->count_matching_subqs() + r->count_matching_subqs());
+}
