@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -148,6 +148,9 @@ DEFINE_TESTCASE(emptyquery1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 0);
 
     vector<Xapian::Query> v;
     enquire.set_query(Xapian::Query(Xapian::Query::OP_AND, v.begin(), v.end()));
@@ -156,6 +159,9 @@ DEFINE_TESTCASE(emptyquery1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 0);
 
     return true;
 }
@@ -398,7 +404,7 @@ DEFINE_TESTCASE(expandweights2, backend) {
 
     Xapian::ESet eset = enquire.get_eset(3, myrset);
     TEST_EQUAL(eset.size(), 3);
-    if (strcmp(get_dbtype(), "multi") != 0) {
+    if (!startswith(get_dbtype(), "multi")) {
 	// For a single database, the weights should be the same with or
 	// without USE_EXACT_TERMFREQ.
 	TEST_EQUAL_DOUBLE(eset[0].get_weight(), 6.08904001099445);
@@ -518,7 +524,6 @@ DEFINE_TESTCASE(topercent2, backend) {
     TEST_EQUAL(mymset.get_matches_upper_bound(), localmset.get_matches_upper_bound());
     TEST_EQUAL(mymset.get_matches_estimated(), localmset.get_matches_estimated());
     TEST_EQUAL_DOUBLE(mymset.get_max_attained(), localmset.get_max_attained());
-    TEST_EQUAL_DOUBLE(mymset.get_max_possible(), localmset.get_max_possible());
     TEST_EQUAL(mymset.size(), localmset.size());
     TEST(mset_range_is_same(mymset, 0, localmset, 0, mymset.size()));
 
@@ -533,21 +538,20 @@ DEFINE_TESTCASE(topercent2, backend) {
     i = mymset.begin();
     TEST(i != mymset.end());
     pct = mymset.convert_to_percent(i);
-    TEST_GREATER(pct, 65);
-    TEST_LESSER(pct, 75);
+    TEST_REL(pct,>,60);
+    TEST_REL(pct,<,76);
 
     ++i;
 
     TEST(i != mymset.end());
     pct = mymset.convert_to_percent(i);
-    TEST_GREATER(pct, 40);
-    TEST_LESSER(pct, 50);
+    TEST_REL(pct,>,40);
+    TEST_REL(pct,<,50);
 
     TEST_EQUAL(mymset.get_matches_lower_bound(), localmset.get_matches_lower_bound());
     TEST_EQUAL(mymset.get_matches_upper_bound(), localmset.get_matches_upper_bound());
     TEST_EQUAL(mymset.get_matches_estimated(), localmset.get_matches_estimated());
     TEST_EQUAL_DOUBLE(mymset.get_max_attained(), localmset.get_max_attained());
-    TEST_EQUAL_DOUBLE(mymset.get_max_possible(), localmset.get_max_possible());
     TEST_EQUAL(mymset.size(), localmset.size());
     TEST(mset_range_is_same(mymset, 0, localmset, 0, mymset.size()));
 
@@ -704,9 +708,40 @@ DEFINE_TESTCASE(pctcutoff2, backend) {
     enquire.set_cutoff(cutoff);
     enquire.set_collapse_key(1234); // Value which is always empty.
 
-    mset = enquire.get_mset(0, 1);
-    TEST_EQUAL(mset.size(), 1);
-    TEST_EQUAL(mset.get_matches_lower_bound(), 1);
+    Xapian::MSet mset2 = enquire.get_mset(0, 1);
+    TEST_EQUAL(mset2.size(), 1);
+    TEST_EQUAL(mset2.get_matches_lower_bound(), 1);
+    TEST_REL(mset2.get_uncollapsed_matches_lower_bound(),>=,1);
+    TEST_REL(mset2.get_uncollapsed_matches_lower_bound(),<=,mset.size());
+    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),>=,mset.size());
+    TEST_REL(mset2.get_uncollapsed_matches_lower_bound(),<=,mset2.get_uncollapsed_matches_estimated());
+    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),>=,mset2.get_uncollapsed_matches_estimated());
+
+    return true;
+}
+
+// Test that the percent cutoff option returns all the answers it should.
+DEFINE_TESTCASE(pctcutoff3, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("this"));
+    Xapian::MSet mset1 = enquire.get_mset(0, 10);
+
+    if (verbose) {
+	tout << "Original mset pcts:";
+	print_mset_percentages(mset1);
+	tout << "\n";
+    }
+
+    int percent = 100;
+    for (Xapian::MSetIterator i = mset1.begin(); i != mset1.end(); ++i) {
+	int new_percent = mset1.convert_to_percent(i);
+	if (new_percent != percent) {
+	    enquire.set_cutoff(percent);
+	    Xapian::MSet mset2 = enquire.get_mset(0, 10);
+	    TEST_EQUAL(mset2.size(), i.get_rank());
+	    percent = new_percent;
+	}
+    }
 
     return true;
 }
@@ -767,7 +802,8 @@ DEFINE_TESTCASE(cutoff1, backend) {
 // tests the allow query terms expand option
 DEFINE_TESTCASE(allowqterms1, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata"));
-    enquire.set_query(Xapian::Query("this"));
+    string term = "paragraph";
+    enquire.set_query(Xapian::Query(term));
 
     Xapian::MSet mymset = enquire.get_mset(0, 10);
     TEST(mymset.size() >= 2);
@@ -780,13 +816,13 @@ DEFINE_TESTCASE(allowqterms1, backend) {
     Xapian::ESet myeset = enquire.get_eset(1000, myrset);
     Xapian::ESetIterator j = myeset.begin();
     for ( ; j != myeset.end(); ++j) {
-        TEST_NOT_EQUAL(*j, "this");
+	TEST_NOT_EQUAL(*j, term);
     }
 
     Xapian::ESet myeset2 = enquire.get_eset(1000, myrset, Xapian::Enquire::INCLUDE_QUERY_TERMS);
     j = myeset2.begin();
     for ( ; j != myeset2.end(); ++j) {
-        if (*j == "this") break;
+	if (*j == term) break;
     }
     TEST(j != myeset2.end());
     return true;
@@ -1387,16 +1423,10 @@ DEFINE_TESTCASE(qterminfo1, backend) {
 
     TEST_EQUAL(mymset1a.get_termfreq(term1),
 	       mymset2a.get_termfreq(term1));
-    TEST_EQUAL(mymset1a.get_termweight(term1),
-	       mymset2a.get_termweight(term1));
     TEST_EQUAL(mymset1a.get_termfreq(term2),
 	       mymset2a.get_termfreq(term2));
-    TEST_EQUAL(mymset1a.get_termweight(term2),
-	       mymset2a.get_termweight(term2));
     TEST_EQUAL(mymset1a.get_termfreq(term3),
 	       mymset2a.get_termfreq(term3));
-    TEST_EQUAL(mymset1a.get_termweight(term3),
-	       mymset2a.get_termweight(term3));
 
     TEST_EQUAL(mymset1a.get_termfreq(term1), 3);
     TEST_EQUAL(mymset1a.get_termfreq(term2), 1);
@@ -1404,11 +1434,16 @@ DEFINE_TESTCASE(qterminfo1, backend) {
 
     TEST_NOT_EQUAL(mymset1a.get_termweight(term1), 0);
     TEST_NOT_EQUAL(mymset1a.get_termweight(term2), 0);
-    // non-existent terms still have weight
-    TEST_NOT_EQUAL(mymset1a.get_termweight(term3), 0);
+    // non-existent terms should have 0 weight.
+    TEST_EQUAL(mymset1a.get_termweight(term3), 0);
 
+    TEST_EQUAL(mymset1a.get_termfreq(stemmer("banana")), 1);
     TEST_EXCEPTION(Xapian::InvalidArgumentError,
-		   mymset1a.get_termfreq("sponge"));
+		   mymset1a.get_termweight(stemmer("banana")));
+
+    TEST_EQUAL(mymset1a.get_termfreq("sponge"), 0);
+    TEST_EXCEPTION(Xapian::InvalidArgumentError,
+		   mymset1a.get_termweight("sponge"));
 
     return true;
 }
@@ -1463,6 +1498,9 @@ DEFINE_TESTCASE(matches1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_OR, "inmemory", "word");
     enquire.set_query(myquery);
@@ -1470,6 +1508,9 @@ DEFINE_TESTCASE(matches1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "inmemory", "word");
     enquire.set_query(myquery);
@@ -1477,6 +1518,9 @@ DEFINE_TESTCASE(matches1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 0);
 
     myquery = query(Xapian::Query::OP_AND, "simple", "word");
     enquire.set_query(myquery);
@@ -1484,6 +1528,9 @@ DEFINE_TESTCASE(matches1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "simple", "word");
     enquire.set_query(myquery);
@@ -1492,19 +1539,28 @@ DEFINE_TESTCASE(matches1, backend) {
     // one sub-database has 3 documents and simple and word both have termfreq
     // of 2, so the matcher can tell at least one document must match!)
     // TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
-    TEST(mymset.get_matches_lower_bound() <= mymset.get_matches_estimated());
+    TEST_REL(mymset.get_matches_lower_bound(),<=,mymset.get_matches_estimated());
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,mymset.get_uncollapsed_matches_estimated());
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     mymset = enquire.get_mset(0, 1);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     mymset = enquire.get_mset(0, 2);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "paragraph", "another");
     enquire.set_query(myquery);
@@ -1512,21 +1568,33 @@ DEFINE_TESTCASE(matches1, backend) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     mymset = enquire.get_mset(0, 1);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     mymset = enquire.get_mset(0, 2);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 1);
 
     mymset = enquire.get_mset(1, 20);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 1);
 
     return true;
 }
@@ -1704,7 +1772,6 @@ DEFINE_TESTCASE(spaceterms1, backend) {
 // test that XOR queries work
 DEFINE_TESTCASE(xor1, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata"));
-    enquire.set_query(Xapian::Query("this"));
     Xapian::Stem stemmer("english");
 
     vector<string> terms;
@@ -1717,11 +1784,44 @@ DEFINE_TESTCASE(xor1, backend) {
     enquire.set_query(query);
 
     Xapian::MSet mymset = enquire.get_mset(0, 10);
+    //	Docid	this	word	of	Match?
+    //	1	*			*
+    //	2	*	*	*	*
+    //	3	*		*
+    //	4	*	*
+    //	5	*			*
+    //	6	*			*
     mset_expect_order(mymset, 1, 2, 5, 6);
 
     return true;
 }
 
+/// Test that weighted XOR queries work (bug fixed in 1.2.1 and 1.0.21).
+DEFINE_TESTCASE(xor2, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    Xapian::Stem stemmer("english");
+
+    vector<string> terms;
+    terms.push_back(stemmer("this"));
+    terms.push_back(stemmer("word"));
+    terms.push_back(stemmer("of"));
+
+    Xapian::Query query(Xapian::Query::OP_XOR, terms.begin(), terms.end());
+    enquire.set_query(query);
+
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
+    //	Docid	LEN	this	word	of	Match?
+    //	1	28	2			*
+    //	2	81	5	8	1	*
+    //	3	15	1		2
+    //	4	31	1	1
+    //	5	15	1			*
+    //	6	15	1			*
+    mset_expect_order(mymset, 2, 1, 5, 6);
+
+    return true;
+}
+
 // test Xapian::Database::get_document()
 DEFINE_TESTCASE(getdoc1, backend) {
     Xapian::Database db(get_database("apitest_onedoc"));
@@ -1774,10 +1874,14 @@ DEFINE_TESTCASE(checkatleast2, backend) {
     Xapian::MSet mymset = enquire.get_mset(0, 3, 10);
     TEST_MSET_SIZE(mymset, 3);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 5);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 5);
 
     mymset = enquire.get_mset(0, 2, 4);
     TEST_MSET_SIZE(mymset, 2);
-    TEST_GREATER_OR_EQUAL(mymset.get_matches_lower_bound(), 4);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,4);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,4);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,4);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,4);
 
     return true;
 }
@@ -1822,16 +1926,23 @@ DEFINE_TESTCASE(checkatleast3, backend) {
 	    TEST_EQUAL(mset.get_matches_lower_bound(), 60);
 	    TEST_EQUAL(mset.get_matches_estimated(), 60);
 	    TEST_EQUAL(mset.get_matches_upper_bound(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_lower_bound(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_estimated(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_upper_bound(), 60);
 
 	    mset = enquire.get_mset(0, 50, 100);
 	    TEST_MSET_SIZE(mset, 50);
 	    TEST_EQUAL(mset.get_matches_lower_bound(), 60);
 	    TEST_EQUAL(mset.get_matches_estimated(), 60);
 	    TEST_EQUAL(mset.get_matches_upper_bound(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_lower_bound(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_estimated(), 60);
+	    TEST_EQUAL(mset.get_uncollapsed_matches_upper_bound(), 60);
 
 	    mset = enquire.get_mset(0, 10, 50);
 	    TEST_MSET_SIZE(mset, 10);
-	    TEST(mset.get_matches_lower_bound() >= 50);
+	    TEST_REL(mset.get_matches_lower_bound(),>=,50);
+	    TEST_REL(mset.get_uncollapsed_matches_lower_bound(),>=,50);
 	}
     }
 
@@ -1892,60 +2003,6 @@ DEFINE_TESTCASE(emptyterm1, backend) {
     return true;
 }
 
-// Feature test for Query::OP_VALUE_RANGE.
-DEFINE_TESTCASE(valuerange1, backend) {
-    Xapian::Database db(get_database("apitest_phrase"));
-    Xapian::Enquire enq(db);
-    static const char * vals[] = {
-	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
-    };
-    for (const char **start = vals; *start; ++start) {
-	for (const char **end = vals; *end; ++end) {
-	    Xapian::Query query(Xapian::Query::OP_VALUE_RANGE, 1, *start, *end);
-	    enq.set_query(query);
-	    Xapian::MSet mset = enq.get_mset(0, 20);
-	    // Check that documents in the MSet match the value range filter.
-	    set<Xapian::docid> matched;
-	    Xapian::MSetIterator i;
-	    for (i = mset.begin(); i != mset.end(); ++i) {
-		matched.insert(*i);
-		string value = db.get_document(*i).get_value(1);
-		tout << "'" << *start << "' <= '" << value << "' <= '" << *end << "'" << endl;
-		TEST(value >= *start);
-		TEST(value <= *end);
-	    }
-	    // Check that documents not in the MSet don't match the value range filter.
-	    for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
-		if (matched.find(j) == matched.end()) {
-		    string value = db.get_document(j).get_value(1);
-		    tout << value << " < '" << *start << "' or > '" << *end << "'" << endl;
-		    TEST(value < *start || value > *end);
-		}
-	    }
-	}
-    }
-    return true;
-}
-
-// Regression test for Query::OP_VALUE_LE - used to return document IDs for
-// non-existent documents.
-DEFINE_TESTCASE(valuerange2, writable) {
-    Xapian::WritableDatabase db = get_writable_database();
-    Xapian::Document doc;
-    doc.set_data("5");
-    doc.add_value(0, "5");
-    db.replace_document(5, doc);
-    Xapian::Enquire enq(db);
-
-    Xapian::Query query(Xapian::Query::OP_VALUE_LE, 0, "6");
-    enq.set_query(query);
-    Xapian::MSet mset = enq.get_mset(0, 20);
-
-    TEST_EQUAL(mset.size(), 1);
-    TEST_EQUAL(*(mset[0]), 5);
-    return true;
-}
-
 // Test for alldocs postlist with a sparse database.
 DEFINE_TESTCASE(alldocspl1, writable) {
     Xapian::WritableDatabase db = get_writable_database();
@@ -1975,7 +2032,7 @@ DEFINE_TESTCASE(alldocspl2, writable) {
 	doc.add_value(0, "5");
 	db.replace_document(5, doc);
 
-	// Test iterating before flushing the changes.
+	// Test iterating before committing the changes.
 	i = db.postlist_begin("");
 	end = db.postlist_end("");
 	TEST(i != end);
@@ -1985,9 +2042,9 @@ DEFINE_TESTCASE(alldocspl2, writable) {
 	++i;
 	TEST(i == end);
 
-	db.flush();
+	db.commit();
 
-	// Test iterating after flushing the changes.
+	// Test iterating after committing the changes.
 	i = db.postlist_begin("");
 	end = db.postlist_end("");
 	TEST(i != end);
@@ -2003,7 +2060,7 @@ DEFINE_TESTCASE(alldocspl2, writable) {
 	doc.add_value(0, "7");
 	db.replace_document(7, doc);
 
-	// Test iterating through before flushing the changes.
+	// Test iterating through before committing the changes.
 	i = db.postlist_begin("");
 	end = db.postlist_end("");
 	TEST(i != end);
@@ -2021,7 +2078,7 @@ DEFINE_TESTCASE(alldocspl2, writable) {
 	// Delete the first document.
 	db.delete_document(5);
 
-	// Test iterating through before flushing the changes.
+	// Test iterating through before committing the changes.
 	i = db.postlist_begin("");
 	end = db.postlist_end("");
 	TEST(i != end);
@@ -2031,8 +2088,9 @@ DEFINE_TESTCASE(alldocspl2, writable) {
 	++i;
 	TEST(i == end);
 
-	// Test iterating through after flushing the changes, and dropping the reference to the main DB.
-	db.flush();
+	// Test iterating through after committing the changes, and dropping the
+	// reference to the main DB.
+	db.commit();
 	i = db.postlist_begin("");
 	end = db.postlist_end("");
     }
@@ -2047,70 +2105,6 @@ DEFINE_TESTCASE(alldocspl2, writable) {
     return true;
 }
 
-// Feature test for Query::OP_VALUE_GE.
-DEFINE_TESTCASE(valuege1, backend) {
-    Xapian::Database db(get_database("apitest_phrase"));
-    Xapian::Enquire enq(db);
-    static const char * vals[] = {
-	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
-    };
-    for (const char **start = vals; *start; ++start) {
-	Xapian::Query query(Xapian::Query::OP_VALUE_GE, 1, *start);
-	enq.set_query(query);
-	Xapian::MSet mset = enq.get_mset(0, 20);
-	// Check that documents in the MSet match the value range filter.
-	set<Xapian::docid> matched;
-	Xapian::MSetIterator i;
-	for (i = mset.begin(); i != mset.end(); ++i) {
-	    matched.insert(*i);
-	    string value = db.get_document(*i).get_value(1);
-	    tout << "'" << *start << "' <= '" << value << "'" << endl;
-	    TEST(value >= *start);
-	}
-	// Check that documents not in the MSet don't match the value range filter.
-	for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
-	    if (matched.find(j) == matched.end()) {
-		string value = db.get_document(j).get_value(1);
-		tout << value << " < '" << *start << "'" << endl;
-		TEST(value < *start);
-	    }
-	}
-    }
-    return true;
-}
-
-// Feature test for Query::OP_VALUE_LE.
-DEFINE_TESTCASE(valuele1, backend) {
-    Xapian::Database db(get_database("apitest_phrase"));
-    Xapian::Enquire enq(db);
-    static const char * vals[] = {
-	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
-    };
-    for (const char **end = vals; *end; ++end) {
-	Xapian::Query query(Xapian::Query::OP_VALUE_LE, 1, *end);
-	enq.set_query(query);
-	Xapian::MSet mset = enq.get_mset(0, 20);
-	// Check that documents in the MSet match the value range filter.
-	set<Xapian::docid> matched;
-	Xapian::MSetIterator i;
-	for (i = mset.begin(); i != mset.end(); ++i) {
-	    matched.insert(*i);
-	    string value = db.get_document(*i).get_value(1);
-	    tout << "'" << *end << "' <= '" << value << "'" << endl;
-	    TEST(value <= *end);
-	}
-	// Check that documents not in the MSet don't match the value range filter.
-	for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
-	    if (matched.find(j) == matched.end()) {
-		string value = db.get_document(j).get_value(1);
-		tout << value << " < '" << *end << "'" << endl;
-		TEST(value > *end);
-	    }
-	}
-    }
-    return true;
-}
-
 // Feature test for Query::OP_SCALE_WEIGHT.
 DEFINE_TESTCASE(scaleweight1, backend) {
     Xapian::Database db(get_database("apitest_phrase"));
@@ -2128,7 +2122,7 @@ DEFINE_TESTCASE(scaleweight1, backend) {
 	"leave \"milk notpresent\"",
 	NULL
     };
-    static double multipliers[] = {
+    static const double multipliers[] = {
 	-1000000, -2.5, -1, -0.5, 0, 0.5, 1, 2.5, 1000000,
 	0, 0
     };
@@ -2136,7 +2130,7 @@ DEFINE_TESTCASE(scaleweight1, backend) {
     for (const char **qstr = queries; *qstr; ++qstr) {
 	Xapian::Query query1 = qp.parse_query(*qstr);
 	tout << "query1: " << query1.get_description() << endl;
-	for (double *multp = multipliers; multp[0] != multp[1]; ++multp) {
+	for (const double *multp = multipliers; multp[0] != multp[1]; ++multp) {
 	    double mult = *multp;
 	    if (mult < 0) {
 		TEST_EXCEPTION(Xapian::InvalidArgumentError,
@@ -2263,3 +2257,32 @@ DEFINE_TESTCASE(tradweight1, backend) {
 
     return true;
 }
+
+// Feature test for Database::get_uuid().
+DEFINE_TESTCASE(uuid1, backend && !multi) {
+    SKIP_TEST_FOR_BACKEND("inmemory");
+    Xapian::Database db = get_database("apitest_simpledata");
+    string uuid1 = db.get_uuid();
+    TEST_EQUAL(uuid1.size(), 36);
+
+    // A database with no sub-databases has an empty UUID.
+    Xapian::Database db2;
+    TEST_EQUAL(string(), db2.get_uuid());
+
+    db2.add_database(db);
+    TEST_EQUAL(uuid1, db2.get_uuid());
+
+    // Multi-database has multiple UUIDs (we don't define the format exactly
+    // so this assumes something about the implementation).
+    db2.add_database(db);
+    TEST_EQUAL(uuid1 + ":" + uuid1, db2.get_uuid());
+
+#ifdef XAPIAN_HAS_INMEMORY_BACKEND
+    // This relies on InMemory databases not supporting uuids.
+    // A multi-database containing a database with no uuid has no uuid.
+    db2.add_database(Xapian::InMemory::open());
+    TEST_EQUAL(string(), db2.get_uuid());
+#endif
+
+    return true;
+}
