@@ -1,5 +1,5 @@
 /** @file api_unicode.cc
- * @brief test the Unicode and UTF-8 classes and functions.
+ * @brief Test the Unicode and UTF-8 classes and functions.
  */
 /* Copyright (C) 2006,2007,2008,2009 Olly Betts
  *
@@ -103,11 +103,19 @@ DEFINE_TESTCASE(unicode1,!backend) {
     TEST_EQUAL(Unicode::get_category(0xa3), Unicode::CURRENCY_SYMBOL);
     // U+0242 was added in Unicode 5.0.0.
     TEST_EQUAL(Unicode::get_category(0x242), Unicode::LOWERCASE_LETTER);
+    // U+11A7 was added in Unicode 5.2.0.
+    TEST_EQUAL(Unicode::get_category(0x11A7), Unicode::OTHER_LETTER);
+    // U+9FCB was added in Unicode 5.2.0.
+    TEST_EQUAL(Unicode::get_category(0x9FCB), Unicode::OTHER_LETTER);
+    // U+FA6C was added in Unicode 5.2.0.
+    TEST_EQUAL(Unicode::get_category(0xFA6C), Unicode::OTHER_LETTER);
     TEST_EQUAL(Unicode::get_category(0xFFFF), Unicode::UNASSIGNED);
     // Test characters outside BMP.
     TEST_EQUAL(Unicode::get_category(0x10345), Unicode::OTHER_LETTER);
     TEST_EQUAL(Unicode::get_category(0x10FFFD), Unicode::PRIVATE_USE);
     TEST_EQUAL(Unicode::get_category(0x10FFFF), Unicode::UNASSIGNED);
+    // U+1109A was added in Unicode 5.2.0.
+    TEST_EQUAL(Unicode::get_category(0x1109a), Unicode::OTHER_LETTER);
     // Test some invalid Unicode values.
     TEST_EQUAL(Unicode::get_category(0x110000), Unicode::UNASSIGNED);
     TEST_EQUAL(Unicode::get_category(0xFFFFFFFF), Unicode::UNASSIGNED);
@@ -157,6 +165,40 @@ DEFINE_TESTCASE(caseconvert1,!backend) {
     return true;
 }
 
+/// Test Unicode 5.1 support.
+DEFINE_TESTCASE(caseconvert2,!backend) {
+    using namespace Xapian;
+
+    TEST_EQUAL(Unicode::toupper(0x250), 0x2c6f);
+    TEST_EQUAL(Unicode::toupper(0x251), 0x2c6d);
+    TEST_EQUAL(Unicode::toupper(0x271), 0x2c6e);
+
+    TEST_EQUAL(Unicode::get_category(0x2ec), Unicode::MODIFIER_LETTER);
+    TEST_EQUAL(Unicode::get_category(0x374), Unicode::MODIFIER_LETTER);
+    TEST_EQUAL(Unicode::get_category(0x487), Unicode::NON_SPACING_MARK);
+    TEST_EQUAL(Unicode::get_category(0x5be), Unicode::DASH_PUNCTUATION);
+    TEST_EQUAL(Unicode::get_category(0x1f093), Unicode::OTHER_SYMBOL);
+
+    TEST_EQUAL(Unicode::tolower(0x370), 0x371);
+    TEST_EQUAL(Unicode::toupper(0x371), 0x370);
+    TEST_EQUAL(Unicode::tolower(0x372), 0x373);
+    TEST_EQUAL(Unicode::toupper(0x373), 0x372);
+    TEST_EQUAL(Unicode::tolower(0x376), 0x377);
+    TEST_EQUAL(Unicode::toupper(0x377), 0x376);
+    TEST_EQUAL(Unicode::tolower(0x3cf), 0x3d7);
+    TEST_EQUAL(Unicode::toupper(0x3d7), 0x3cf);
+
+    unsigned u;
+    for (u = 0x514; u < 0x524; u += 2) {
+	TEST_EQUAL(Unicode::get_category(u), Unicode::UPPERCASE_LETTER);
+	TEST_EQUAL(Unicode::get_category(u + 1), Unicode::LOWERCASE_LETTER);
+	TEST_EQUAL(Unicode::tolower(u), u + 1);
+	TEST_EQUAL(Unicode::toupper(u + 1), u);
+    }
+	
+    return true;
+}
+
 DEFINE_TESTCASE(utf8convert1,!backend) {
     string s;
     Xapian::Unicode::append_utf8(s, 'a');
@@ -187,12 +229,23 @@ DEFINE_TESTCASE(unicodepredicates1,!backend) {
 	// DECIMAL_DIGIT_NUMER
 	'0', '7', '9',
 	// LOWERCASE_LETTER
-	'a', 'z', 0x242, 0x250, 0x251, 0x271, 0x3d7,
+	'a', 'z', 0x250, 0x251, 0x271, 0x3d7,
+	0x242, // (added in Unicode 5.0.0)
+	// LOWERCASE_LETTER (added in Unicode 5.1.0)
+	0x371, 0x373, 0x377, 0x514, 0x516, 0x518, 0x51a, 0x51c, 0x51e,
+	0x520, 0x522,
 	// UPPERCASE_LETTER
 	'A', 'Z', 0x241,
+	// UPPERCASE_LETTER (added in Unicode 5.1.0)
+	0x370, 0x372, 0x376, 0x3cf, 0x515, 0x517, 0x519, 0x51b, 0x51d, 0x51f,
+	0x521, 0x523, 0x2c6d, 0x2c6e, 0x2c6f,
 	// OTHER_LETTER
 	0x10345,
-	// MODIFIER_LETTER
+	// MODIFIER_LETTER (added in Unicode 5.1.0)
+	0x2ec, 0x374,
+	// NON_SPACING_MARK (added to is_wordchar() in 1.1.0)
+	0x651,
+	0x487, // Added in Unicode 5.1.0
 	0
     };
     const unsigned currency[] = {
@@ -201,52 +254,43 @@ DEFINE_TESTCASE(unicodepredicates1,!backend) {
 	0
     };
     const unsigned whitespace[] = {
-       	// CONTROL
+	// CONTROL
 	'\t', '\n', '\f', '\r',
 	// SPACE_SEPARATOR
-       	' ',
+	' ',
 	0
     };
     const unsigned other[] = {
-       	// DASH_PUNCTUATION
-       	// OTHER_SYMBOL
-       	// UNASSIGNED
+	// DASH_PUNCTUATION (added in Unicode 5.1.0)
+	0x5be,
+	// OTHER_SYMBOL (added in Unicode 5.1.0)
+	0x1f093,
+	// UNASSIGNED
 	0xffff, 0x10ffff, 0x110000, 0xFFFFFFFF,
-       	// PRIVATE_USE
+	// PRIVATE_USE
 	0x10fffd,
-	// NON_SPACING_MARK
-	0x651,
-	// New in Unicode 5.1.0 (Xapian 1.0.x uses Unicode 5.0.0)
-	0x2ec, 0x370, 0x371, 0x372, 0x373, 0x374, 0x376, 0x377, 0x3cf,
-	0x487, 0x514, 0x515, 0x516, 0x517, 0x518, 0x519, 0x51a, 0x51b, 0x51c,
-	0x51d, 0x51e, 0x51f, 0x520, 0x521, 0x522, 0x523, 0x5be,
-	0x2c6d, 0x2c6e, 0x2c6f, 0x1f093,
 	0
     };
 
     for (const unsigned * p = wordchars; *p; ++p) {
-	tout << *p << endl;
 	TEST(Xapian::Unicode::is_wordchar(*p));
 	TEST(!Xapian::Unicode::is_currency(*p));
 	TEST(!Xapian::Unicode::is_whitespace(*p));
     }
 
     for (const unsigned * p = currency; *p; ++p) {
-	tout << *p << endl;
 	TEST(!Xapian::Unicode::is_wordchar(*p));
 	TEST(Xapian::Unicode::is_currency(*p));
 	TEST(!Xapian::Unicode::is_whitespace(*p));
     }
 
     for (const unsigned * p = whitespace; *p; ++p) {
-	tout << *p << endl;
 	TEST(!Xapian::Unicode::is_wordchar(*p));
 	TEST(!Xapian::Unicode::is_currency(*p));
 	TEST(Xapian::Unicode::is_whitespace(*p));
     }
 
     for (const unsigned * p = other; *p; ++p) {
-	tout << *p << endl;
 	TEST(!Xapian::Unicode::is_wordchar(*p));
 	TEST(!Xapian::Unicode::is_currency(*p));
 	TEST(!Xapian::Unicode::is_whitespace(*p));
