@@ -1,7 +1,8 @@
 /** @file backendmanager_remoteprog.cc
  * @brief BackendManager subclass for remoteprog databases.
  */
-/* Copyright (C) 2007,2009 Olly Betts
+/* Copyright (C) 2007,2008,2009 Olly Betts
+ * Copyright (C) 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,45 +33,25 @@
 
 using namespace std;
 
-const char *
+std::string
 BackendManagerRemoteProg::get_dbtype() const
 {
-    return "remoteprog";
+    return "remoteprog_" + remote_type;
 }
 
 Xapian::Database
-BackendManagerRemoteProg::get_database(const vector<string> & files)
+BackendManagerRemoteProg::do_get_database(const vector<string> & files)
 {
     // Default to a long (5 minute) timeout so that tests won't fail just
     // because the host is slow or busy.
     return BackendManagerRemoteProg::get_remote_database(files, 300000);
 }
 
-Xapian::Database
-BackendManagerRemoteProg::get_database(const string & file)
-{
-    return BackendManagerRemoteProg::get_database(vector<string>(1, file));
-}
-
 Xapian::WritableDatabase
 BackendManagerRemoteProg::get_writable_database(const string & name,
 						const string & file)
 {
-    last_wdb_name = name;
-
-    vector<string> files(1, file);
-    // Default to a long (5 minute) timeout so that tests won't fail just
-    // because the host is slow or busy.
-    string args = "-t300000 --writable ";
-
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    (void)getwritedb_flint(name, files);
-    args += ".flint/";
-#else
-    (void)getwritedb_quartz(name, files);
-    args += ".quartz/";
-#endif
-    args += name;
+    string args = get_writable_database_args(name, file);
 
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
@@ -85,14 +66,8 @@ Xapian::Database
 BackendManagerRemoteProg::get_remote_database(const vector<string> & files,
 					      unsigned int timeout)
 {
-    string args = "-t";
-    args += om_tostring(timeout);
-    args += ' ';
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    args += createdb_flint(files);
-#else
-    args += createdb_quartz(files);
-#endif
+    string args = get_remote_database_args(files, timeout);
+
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
 	args.insert(0, XAPIAN_PROGSRV" ");
@@ -105,13 +80,7 @@ BackendManagerRemoteProg::get_remote_database(const vector<string> & files,
 Xapian::Database
 BackendManagerRemoteProg::get_writable_database_as_database()
 {
-    string args = "-t300000 ";
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    args += ".flint/";
-#else
-    args += ".quartz/";
-#endif
-    args += last_wdb_name;
+    string args = get_writable_database_as_database_args();
 
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
@@ -125,13 +94,7 @@ BackendManagerRemoteProg::get_writable_database_as_database()
 Xapian::WritableDatabase
 BackendManagerRemoteProg::get_writable_database_again()
 {
-    string args = "-t300000 --writable ";
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    args += ".flint/";
-#else
-    args += ".quartz/";
-#endif
-    args += last_wdb_name;
+    string args = get_writable_database_again_args();
 
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
