@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,10 +24,13 @@
 
 #include "testsuite.h"
 
+#include "backendmanager.h"
+#include "fdtracker.h"
+#include "testrunner.h"
+
 #ifdef HAVE_VALGRIND
 # include "safeerrno.h"
 # include <valgrind/memcheck.h>
-# include <stdio.h>
 # include <sys/types.h>
 # include "safefcntl.h"
 # include "safeunistd.h"
@@ -35,36 +38,29 @@
 
 #include <algorithm>
 #include <iostream>
-
-#ifdef HAVE_STREAMBUF
-#include <streambuf>
-#else // HAVE_STREAMBUF
-#include <streambuf.h>
-#endif // HAVE_STREAMBUF
-
 #include <set>
 
-#include <float.h> // For DBL_DIG.
-#include <math.h> // For ceil, fabs, log10.
-#include <stdlib.h>
-#include <string.h>
+#include <cfloat> // For DBL_DIG.
+#include <cmath> // For ceil, fabs, log10.
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
 #include "gnu_getopt.h"
 
-#include <setjmp.h>
+#include <csetjmp>
 #include <signal.h>
 
 #include <exception>
 #ifdef USE_RTTI
 # include <typeinfo>
-# if defined __GNUC__ && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
-#  include <cxxabi.h>
+# ifdef __GNUC__
+#  include <cxxabi.h> // Added in GCC 3.1 which is now required.
 # endif
 #endif
 
 #include <xapian/error.h>
 #include "noreturn.h"
-#include "omdebug.h"
 #include "stringutils.h"
 #include "utils.h"
 
@@ -77,6 +73,16 @@ bool verbose;
 static int vg_log_fd = -1;
 #endif
 
+#ifdef HAVE_SIGSETJMP
+# define SIGSETJMP(ENV, SAVESIGS) sigsetjmp(ENV, SAVESIGS)
+# define SIGLONGJMP(ENV, VAL) siglongjmp(ENV, VAL)
+# define SIGJMP_BUF sigjmp_buf
+#else
+# define SIGSETJMP(ENV, SAVESIGS) setjmp(ENV)
+# define SIGLONGJMP(ENV, VAL) longjmp(ENV, VAL)
+# define SIGJMP_BUF jmp_buf
+#endif
+
 /// The exception type we were expecting in TEST_EXCEPTION
 //  We use this to attempt to diagnose when the code fails to catch an
 //  exception when it should (due to a compiler or runtime fault in
@@ -161,15 +167,46 @@ test_driver::test_driver(const test_desc *tests_)
 {
 }
 
-static jmp_buf jb;
+static SIGJMP_BUF jb;
 static int signum = 0;
+static void * sigaddr = NULL;
 
-/* Needs C linkage so we can pass it to signal() without problems. */
+// Needs C linkage so we can pass it to sigaction()/signal() without problems.
 extern "C" {
 
+#if defined HAVE_SIGACTION && defined SA_SIGINFO
+XAPIAN_NORETURN(static void handle_sig(int signum_, siginfo_t *si, void *));
+static void handle_sig(int signum_, siginfo_t *si, void *)
+{
+    // Disable all our signal handlers to avoid problems if the signal
+    // handling code causes a signal.
+    struct sigaction sa;
+    sa.sa_handler = SIG_DFL;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = 0;
+    // We set the handlers with SA_RESETHAND, but that will only reset the
+    // handler for the signal which fired.
+    if (signum_ != SIGSEGV) sigaction(SIGSEGV, &sa, NULL);
+    if (signum_ != SIGFPE) sigaction(SIGFPE, &sa, NULL);
+    if (signum_ != SIGILL) sigaction(SIGILL, &sa, NULL);
+# ifdef SIGBUS
+    if (signum_ != SIGBUS) sigaction(SIGBUS, &sa, NULL);
+# endif
+# ifdef SIGSTKFLT
+    if (signum_ != SIGSTKFLT) sigaction(SIGSTKFLT, &sa, NULL);
+# endif
+    signum = signum_;
+    sigaddr = si->si_addr;
+    SIGLONGJMP(jb, 1);
+}
+
+#else
+
 XAPIAN_NORETURN(static void handle_sig(int signum_));
 static void handle_sig(int signum_)
 {
+    // Disable all our signal handlers to avoid problems if the signal
+    // handling code causes a signal.
     signal(SIGSEGV, SIG_DFL);
     signal(SIGFPE, SIG_DFL);
     signal(SIGILL, SIG_DFL);
@@ -180,8 +217,9 @@ static void handle_sig(int signum_)
     signal(SIGSTKFLT, SIG_DFL);
 #endif
     signum = signum_;
-    longjmp(jb, 1);
+    SIGLONGJMP(jb, 1);
 }
+#endif
 
 }
 
@@ -192,26 +230,61 @@ class SignalRedirector {
     SignalRedirector() : active(false) { }
     void activate() {
 	active = true;
+	signum = 0;
+	sigaddr = NULL;
+	// SA_SIGINFO not universal (e.g. not present on Linux < 2.2 and Hurd).
+#if defined HAVE_SIGACTION && defined SA_SIGINFO
+	struct sigaction sa;
+	sa.sa_sigaction = handle_sig;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = SA_RESETHAND|SA_SIGINFO;
+	sigaction(SIGSEGV, &sa, NULL);
+	sigaction(SIGFPE, &sa, NULL);
+	sigaction(SIGILL, &sa, NULL);
+# ifdef SIGBUS
+	sigaction(SIGBUS, &sa, NULL);
+# endif
+# ifdef SIGSTKFLT
+	sigaction(SIGSTKFLT, &sa, NULL);
+# endif
+#else
 	signal(SIGSEGV, handle_sig);
 	signal(SIGFPE, handle_sig);
 	signal(SIGILL, handle_sig);
-#ifdef SIGBUS
+# ifdef SIGBUS
 	signal(SIGBUS, handle_sig);
-#endif
-#ifdef SIGSTKFLT
+# endif
+# ifdef SIGSTKFLT
 	signal(SIGSTKFLT, handle_sig);
+# endif
 #endif
     }
     ~SignalRedirector() {
 	if (active) {
+#if defined HAVE_SIGACTION && defined SA_SIGINFO
+	    struct sigaction sa;
+	    sa.sa_handler = SIG_DFL;
+	    sigemptyset(&sa.sa_mask);
+	    sa.sa_flags = 0;
+	    sigaction(SIGSEGV, &sa, NULL);
+	    sigaction(SIGFPE, &sa, NULL);
+	    sigaction(SIGILL, &sa, NULL);
+# ifdef SIGBUS
+	    sigaction(SIGBUS, &sa, NULL);
+# endif
+# ifdef SIGSTKFLT
+	    sigaction(SIGSTKFLT, &sa, NULL);
+# endif
+#else
 	    signal(SIGSEGV, SIG_DFL);
 	    signal(SIGFPE, SIG_DFL);
 	    signal(SIGILL, SIG_DFL);
-#ifdef SIGBUS
+# ifdef SIGBUS
 	    signal(SIGBUS, SIG_DFL);
-#endif
-#ifdef SIGSTKFLT
+# endif
+# ifdef SIGSTKFLT
 	    signal(SIGSTKFLT, SIG_DFL);
+# endif
 #endif
 	}
     }
@@ -226,15 +299,16 @@ class SignalRedirector {
 test_driver::test_result
 test_driver::runtest(const test_desc *test)
 {
-#ifdef HAVE_VALGRIND
     // This is used to make a note of how many times we've run the test
     volatile int runcount = 0;
-#endif
+
+    FDTracker fdtracker;
+    fdtracker.init();
 
     while (true) {
 	tout.str(string());
-	SignalRedirector sig; // use object so signal handlers are reset
-	if (!setjmp(jb)) {
+	if (SIGSETJMP(jb, 1) == 0) {
+	    SignalRedirector sig; // use object so signal handlers are reset
 	    static bool catch_signals =
 		(getenv("XAPIAN_TESTSUITE_SIG_DFL") == NULL);
 	    if (catch_signals) sig.activate();
@@ -257,6 +331,8 @@ test_driver::runtest(const test_desc *test)
 		    write_and_clear_tout();
 		    return FAIL;
 		}
+		if (backendmanager)
+		    backendmanager->clean_up();
 #ifdef HAVE_VALGRIND
 		if (vg_log_fd != -1) {
 		    // We must empty tout before asking valgrind to perform its
@@ -347,6 +423,9 @@ test_driver::runtest(const test_desc *test)
 			if (runcount == 0) {
 			    out << col_yellow << " PROBABLY LEAKED MEMORY - RETRYING TEST" << col_reset;
 			    ++runcount;
+			    // Ensure that any cached memory from fd tracking
+			    // is allocated before we rerun the test.
+			    (void)fdtracker.check();
 			    continue;
 			}
 			REPORT_FAIL_VG("PROBABLY LEAKED " << vg_dubious << " BYTES");
@@ -368,6 +447,9 @@ test_driver::runtest(const test_desc *test)
 			if (runcount == 0) {
 			    out << col_yellow << " POSSIBLE UNRELEASED MEMORY - RETRYING TEST" << col_reset;
 			    ++runcount;
+			    // Ensure that any cached memory from fd tracking
+			    // is allocated before we rerun the test.
+			    (void)fdtracker.check();
 			    continue;
 			}
 			REPORT_FAIL_VG("FAILED TO RELEASE " << vg_reachable << " BYTES");
@@ -375,6 +457,15 @@ test_driver::runtest(const test_desc *test)
 		    }
 		}
 #endif
+		if (!fdtracker.check()) {
+		    if (runcount == 0) {
+			out << col_yellow << " POSSIBLE FDLEAK:" << fdtracker.get_message() << col_reset;
+			++runcount;
+			continue;
+		    }
+		    out << col_red << " FDLEAK:" << fdtracker.get_message() << col_reset;
+		    return FAIL;
+		}
 	    } catch (const TestFail &) {
 		out << col_red << " FAILED" << col_reset;
 		write_and_clear_tout();
@@ -402,7 +493,7 @@ test_driver::runtest(const test_desc *test)
 		out << "std::exception";
 #else
 		const char * name = typeid(e).name();
-# if defined __GNUC__ && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
+# ifdef __GNUC__
 		// __cxa_demangle() apparently requires GCC >= 3.1.
 		// Demangle the name which GCC returns for type_info::name().
 		int status;
@@ -420,6 +511,16 @@ test_driver::runtest(const test_desc *test)
 		out << ": " << e.what() << col_reset;
 		write_and_clear_tout();
 		return FAIL;
+	    } catch (const char * msg) {
+		out << col_red;
+		if (msg) {
+		    out << " EXCEPTION char * " << msg;
+		} else {
+		    out << " EXCEPTION (char*)NULL";
+		}
+		out << col_reset;
+		write_and_clear_tout();
+		return FAIL;
 	    } catch (...) {
 		out << col_red << " UNKNOWN EXCEPTION" << col_reset;
 		write_and_clear_tout();
@@ -430,6 +531,7 @@ test_driver::runtest(const test_desc *test)
 
 	// Caught a signal.
 	const char *signame = "SIGNAL";
+	bool show_addr = true;
 	switch (signum) {
 	    case SIGSEGV: signame = "SIGSEGV"; break;
 	    case SIGFPE: signame = "SIGFPE"; break;
@@ -438,10 +540,19 @@ test_driver::runtest(const test_desc *test)
 	    case SIGBUS: signame = "SIGBUS"; break;
 #endif
 #ifdef SIGSTKFLT
-	    case SIGSTKFLT: signame = "SIGSTKFLT"; break;
+	    case SIGSTKFLT:
+		signame = "SIGSTKFLT";
+		show_addr = false;
+		break;
 #endif
 	}
-	out << " " << col_red << signame << col_reset;
+	out << " " << col_red << signame;
+	if (show_addr) {
+	    char buf[40];
+	    sprintf(buf, " at %p", sigaddr);
+	    out << buf;
+	}
+	out << col_reset;
 	write_and_clear_tout();
 	return FAIL;
     }
@@ -488,7 +599,10 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
 	if (do_this_test) {
 	    out << "Running test: " << test->name << "...";
 	    out.flush();
-	    switch (runtest(test)) {
+	    test_driver::test_result test_res = runtest(test);
+	    if (backendmanager)
+		backendmanager->clean_up();
+	    switch (test_res) {
 		case PASS:
 		    ++res.succeeded;
 		    if (verbose || !use_cr) {
@@ -501,8 +615,7 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
 		    ++res.failed;
 		    out << endl;
 		    if (abort_on_error) {
-			out << "Test failed - aborting further tests." << endl;
-			return res;
+			throw "Test failed - aborting further tests";
 		    }
 		    break;
 		case SKIP:
@@ -652,14 +765,6 @@ test_driver::parse_command_line(int argc, char **argv)
 	}
     }
 #endif
-
-#ifdef XAPIAN_DEBUG_VERBOSE
-    // We need to display something before we start, or the allocation
-    // made when the first debug message is displayed is (wrongly) picked
-    // up on as a memory leak.
-    DEBUGLINE(UNKNOWN, "Starting testsuite run.");
-    om_debug.initialise();
-#endif /* XAPIAN_DEBUG_VERBOSE */
 }
 
 int
