@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2006,2007,2009 Olly Betts
+ * Copyright 2002,2003,2006,2007,2008,2009,2010 Olly Betts
  * Copyright 2006 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -23,13 +23,12 @@
 
 #include <config.h>
 
-// Need to call Error::get_errno() for testing.
-#define XAPIAN_DEPRECATED(D) D
 #include <xapian.h>
 
-#include <float.h>
+#include <cfloat>
 #include "safeerrno.h"
 
+#include <iostream>
 #include <string>
 
 using namespace std;
@@ -38,10 +37,12 @@ using namespace std;
 #include "testsuite.h"
 #include "testutils.h"
 
+#include "omassert.h"
+#include "omqueryinternal.h"
+#include "pack.h"
 #include "serialise.h"
 #include "serialise-double.h"
-#include "omqueryinternal.h"
-#include "utils.h"
+#include "str.h"
 
 static bool test_except1()
 {
@@ -71,7 +72,7 @@ static bool test_exception1()
 	    }
 	    throw;
 	}
-    } catch (Test_Exception & e) {
+    } catch (const Test_Exception & e) {
 	TEST_EQUAL(e.value, 1);
 	return true;
     }
@@ -110,19 +111,19 @@ static bool test_refcnt1()
 	Xapian::Internal::RefCntPtr<test_refcnt> rcp(p);
 
 	TEST_EQUAL(rcp->ref_count, 1);
-	
+
 	{
 	    Xapian::Internal::RefCntPtr<test_refcnt> rcp2;
 	    rcp2 = rcp;
 	    TEST_EQUAL(rcp->ref_count, 2);
 	    // rcp2 goes out of scope here
 	}
-	
+
 	TEST_AND_EXPLAIN(!deleted, "Object prematurely deleted!");
 	TEST_EQUAL(rcp->ref_count, 1);
 	// rcp goes out of scope here
     }
-    
+
     TEST_AND_EXPLAIN(deleted, "Object not properly deleted");
 
     return true;
@@ -137,9 +138,9 @@ static bool test_refcnt2()
     test_refcnt *p = new test_refcnt(deleted);
 
     Xapian::Internal::RefCntPtr<test_refcnt> rcp(p);
-    
+
     rcp = rcp;
-    
+
     TEST_AND_EXPLAIN(!deleted, "Object deleted by self-assignment");
 
     return true;
@@ -179,7 +180,7 @@ static bool test_autoptr1()
     }
 
     TEST(deleted);
-    
+
     return true;
 }
 
@@ -226,25 +227,25 @@ static bool test_stringcomp1()
 
 static bool test_tostring1()
 {
-    TEST_EQUAL(om_tostring(0), "0");
-    TEST_EQUAL(om_tostring(10), "10");
-    TEST_EQUAL(om_tostring(10u), "10");
-    TEST_EQUAL(om_tostring(-10), "-10");
-    TEST_EQUAL(om_tostring(0xffffffff), "4294967295");
-    TEST_EQUAL(om_tostring(0x7fffffff), "2147483647");
-    TEST_EQUAL(om_tostring(0x7fffffffu), "2147483647");
-    TEST_EQUAL(om_tostring(-0x7fffffff), "-2147483647");
+    TEST_EQUAL(str(0), "0");
+    TEST_EQUAL(str(10), "10");
+    TEST_EQUAL(str(10u), "10");
+    TEST_EQUAL(str(-10), "-10");
+    TEST_EQUAL(str(0xffffffff), "4294967295");
+    TEST_EQUAL(str(0x7fffffff), "2147483647");
+    TEST_EQUAL(str(0x7fffffffu), "2147483647");
+    TEST_EQUAL(str(-0x7fffffff), "-2147483647");
 
 #ifdef __WIN32__
     /* Test the 64 bit integer conversion to string.
      * (Currently only exists for windows.)
      */
-    TEST_EQUAL(om_tostring(10ll), "10");
-    TEST_EQUAL(om_tostring(-10ll), "-10");
-    TEST_EQUAL(om_tostring(0x200000000ll), "8589934592");
+    TEST_EQUAL(str(10ll), "10");
+    TEST_EQUAL(str(-10ll), "-10");
+    TEST_EQUAL(str(0x200000000ll), "8589934592");
 // We don't currently have an "unsigned long long" version since it's not required
 // anywhere in the library.
-//    TEST_EQUAL(om_tostring(0x200000000ull), "8589934592");
+//    TEST_EQUAL(str(0x200000000ull), "8589934592");
 #endif
 
     return true;
@@ -424,8 +425,9 @@ static void
 serialisequery1_helper(const Xapian::Query & query)
 {
     string before = query.internal->serialise();
+    Xapian::Registry reg;
     Xapian::Query::Internal * qint;
-    qint = Xapian::Query::Internal::unserialise(before);
+    qint = Xapian::Query::Internal::unserialise(before, reg);
     string after = qint->serialise();
     delete qint;
     TEST(before == after);
@@ -470,7 +472,6 @@ static bool test_serialiseerror1()
     // get_description() if they hadn't already been converted.
     TEST_STRINGS_EQUAL(e.get_description(), "DatabaseOpeningError: Failed to open database (" + enoent_msg + ")");
 
-    TEST_EQUAL(e.get_errno(), ENOENT);
     TEST_STRINGS_EQUAL(e.get_error_string(), enoent_msg);
 
     string serialisation = serialise_error(e);
@@ -482,8 +483,7 @@ static bool test_serialiseerror1()
     try {
 	// unserialise_error throws an exception.
 	unserialise_error(serialisation, "", "");
-    } catch (Xapian::Error & ecaught) {
-	TEST_EQUAL(ecaught.get_errno(), 0); // errno values aren't portable.
+    } catch (const Xapian::Error & ecaught) {
 	TEST_STRINGS_EQUAL(ecaught.get_error_string(), enoent_msg);
 	threw = true;
     }
@@ -525,14 +525,91 @@ static bool test_temporarydtor1()
     return true;
 }
 
+static bool test_static_assert1()
+{
+    STATIC_ASSERT(true);
+    STATIC_ASSERT(1);
+    STATIC_ASSERT(-1);
+    STATIC_ASSERT(42);
+    STATIC_ASSERT(sizeof(char) == 1);
+
+    // FIXME: We should test cases which should fail, but these are hard to
+    // check with our current test framework.
+
+    STATIC_ASSERT_UNSIGNED_TYPE(bool);
+    STATIC_ASSERT_UNSIGNED_TYPE(unsigned char);
+    STATIC_ASSERT_UNSIGNED_TYPE(unsigned short);
+    STATIC_ASSERT_UNSIGNED_TYPE(unsigned int);
+    STATIC_ASSERT_UNSIGNED_TYPE(unsigned long);
+
+    // FIXME: We should test cases which should fail, but these are hard to
+    // check with our current test framework.
+
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned long, unsigned long);
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned int, unsigned int);
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned short, unsigned short);
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned char, unsigned char);
+
+    STATIC_ASSERT_TYPE_DOMINATES(long, long);
+    STATIC_ASSERT_TYPE_DOMINATES(int, int);
+    STATIC_ASSERT_TYPE_DOMINATES(short, short);
+    STATIC_ASSERT_TYPE_DOMINATES(signed char, signed char);
+
+    STATIC_ASSERT_TYPE_DOMINATES(char, char);
+
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned long, unsigned int);
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned int, unsigned short);
+    STATIC_ASSERT_TYPE_DOMINATES(unsigned short, unsigned char);
+
+    STATIC_ASSERT_TYPE_DOMINATES(long, int);
+    STATIC_ASSERT_TYPE_DOMINATES(int, short);
+    STATIC_ASSERT_TYPE_DOMINATES(short, signed char);
+
+    STATIC_ASSERT_TYPE_DOMINATES(long, unsigned char);
+    STATIC_ASSERT_TYPE_DOMINATES(int, unsigned char);
+    STATIC_ASSERT_TYPE_DOMINATES(short, unsigned char);
+
+    // FIXME: We should test cases which should fail, but these are hard to
+    // check with our current test framework.
+
+    return true;
+}
+
+/// Regression test for bug fixed in 1.1.1.
+static bool test_strbool1()
+{
+    TEST_EQUAL(str(true), "1");
+    TEST_EQUAL(str(false), "0");
+    return true;
+}
+
+/// Test pack_uint_preserving_sort()
+static bool test_pack_uint_preserving_sort1()
+{
+    string prev_packed;
+    for (unsigned int i = 0; i != 1000; ++i) {
+	string packed;
+	pack_uint_preserving_sort(packed, i);
+	const char * ptr = packed.data();
+	const char * end = ptr + packed.size();
+	unsigned int result;
+	TEST(unpack_uint_preserving_sort(&ptr, end, &result));
+	TEST_EQUAL(result, i);
+	TEST(ptr == end);
+	TEST_REL(prev_packed, <, packed);
+	swap(prev_packed, packed);
+    }
+    return true;
+}
+
 // ##################################################################
-// # End of actual tests                                            #
+// # End of actual tests					    #
 // ##################################################################
 
 /// The lists of tests to perform
-test_desc tests[] = {
+static const test_desc tests[] = {
     {"except1",			test_except1},
-    {"exception1",              test_exception1},
+    {"exception1",		test_exception1},
     {"refcnt1",			test_refcnt1},
     {"refcnt2",			test_refcnt2},
     {"autoptr1",		test_autoptr1},
@@ -547,11 +624,17 @@ test_desc tests[] = {
     {"serialisequery1",		test_serialisequery1},
     {"serialiseerror1",		test_serialiseerror1},
 #endif
+    {"static_assert1",		test_static_assert1},
+    {"strbool1",		test_strbool1},
+    {"pack1",			test_pack_uint_preserving_sort1},
     {0, 0}
 };
 
 int main(int argc, char **argv)
-{
+try {
     test_driver::parse_command_line(argc, argv);
     return test_driver::run(tests);
+} catch (const char * e) {
+    cout << e << endl;
+    return 1;
 }
